#ifndef __ASM_ATHEROS_WMAC_SUPPORT_H
#define __ASM_ATHEROS_WMAC_SUPPORT_H
/*
 * These are definitions and functions provided by all Atheros
 * platforms.  It is intended to be a generic support layer that
 * provides access to facilities that would be needed by any
 * wireless implementation.
 *
 * We avoid references to OS-specific type definitions.  That
 * allows a wireless implementation that uses this API to be
 * written in an OS-independent way.
 *
 * Functions provided by the WMAC Support Layer are independent
 * of the "attachment mechansim" -- the way in which the WMAC
 * hardware is attached to the system.  In other words, there
 * are no references to "PCI things" or to "AR5312 things" in
 * this interface.  The wmacsl_id_t contains sufficient information
 * to access the underlying hardware.
 */

struct WMACsl_id_s;
/* WMACsl_id_t * is an opaque pointer to Wireless MAC Support Layer object */
typedef struct WMACsl_id_s WMACsl_id_t;

typedef unsigned long WMACsl_dma_addr_t;

extern unsigned int WMACsl_LineSizeGet(WMACsl_id_t *wmacsl_id);

extern void WMACsl_Reset(WMACsl_id_t *wmacsl_id, unsigned int RCMask);
extern void WMACsl_UnReset(WMACsl_id_t *wmacsl_id);

/* Reset Control mask bits */
#define	AR_RC_PCU		0x00000001	/* PCU Warm Reset */
#define	AR_RC_DMA		0x00000002	/* DMA Warm Reset */
#define	AR_RC_MAC		0x00000004	/* MAC Warm Reset */
#define	AR_RC_PHY		0x00000008	/* PHY Warm Reset */
#define	AR_RC_PCI		0x00000010	/* PCI Core Warm Reset */
#define AR_RC_BB                0x00000020      /* Baseband Warm Reset */
#define	AR_RC_BITS	"\20\1RPCU\2RDMA\3RMAC\4RPHY\5RPCI"

extern void WMACsl_EepromRead(WMACsl_id_t *wmacsl_id,
                              unsigned int off,
                              unsigned int nbytes,
                              char *data);

extern void WMACsl_EepromWrite(WMACsl_id_t *wmacsl_id,
                               unsigned int off,
                               unsigned int nbytes,
                               char *data);

extern void WMACsl_GpioCfgOutput(WMACsl_id_t *wmacsl_id, int gpio);

extern void WMACsl_GpioCfgInput(WMACsl_id_t *wmacsl_id, int gpio);

extern void WMACsl_GpioCfgIntr(WMACsl_id_t *wmacsl_id, int gpio, int ilevel);

extern void WMACsl_GpioSet(WMACsl_id_t *wmacsl_id, int gpio, int val);

extern int WMACsl_GpioGet(WMACsl_id_t *wmacsl_id, int gpio);

enum WMACslDmaDirection {
    WMACSL_DMA_TO_DEVICE,
    WMACSL_DMA_FROM_DEVICE
};

extern WMACsl_dma_addr_t WMACsl_MapSingle(WMACsl_id_t *wmacsl_id,
                                          void *data,
                                          unsigned int length,
                                          enum WMACslDmaDirection direc);

extern void WMACsl_UnmapSingle(WMACsl_id_t *wmacsl_id,
                               WMACsl_dma_addr_t dma_addr,
                               unsigned int length,
                               enum WMACslDmaDirection direc);

extern void WMACsl_DmaSyncSingle(WMACsl_id_t *wmacsl_id,
                                 WMACsl_dma_addr_t dma_addr,
                                 unsigned int length,
                                 enum WMACslDmaDirection direc);

extern void * WMACsl_AllocConsistent(WMACsl_id_t *wmacsl_id,
                                     unsigned int length,
                                     WMACsl_dma_addr_t *dma_addrp);

extern void WMACsl_FreeConsistent(WMACsl_id_t *wmacsl_id,
                                  unsigned int length,
                                  void *desc,
                                  WMACsl_dma_addr_t dma_addr);

extern int WMACsl_StateIsSleeping(WMACsl_id_t *wmacsl_id);

enum WMACslPowerMode {
    WMACSL_POWER_MODE_FORCE_SLEEP  = 0,
    WMACSL_POWER_MODE_FORCE_WAKE   = 1,
    WMACSL_POWER_MODE_NORMAL       = 2,
};

extern enum WMACslPowerMode WMACsl_PowerModeGet(WMACsl_id_t *wmacsl_id);

extern void WMACsl_DMAIn(WMACsl_id_t *wmacsl_id,
                         void *data,
                         unsigned int length);

extern char * WMACsl_mac_address_get(WMACsl_id_t *wmacsl_id);


extern void * WMACsl_MemStartGet(WMACsl_id_t *wmacsl_id);

extern void WMACsl_WMACRevInfoGet(WMACsl_id_t *wmacsl_id,
                                  unsigned int *WMACVersion,
                                  unsigned int *WMACRevision);

extern unsigned long WMACsl_MemSizeGet(WMACsl_id_t *wmacsl_id);



/* These are AR5312-specific functions that support wireless */

extern unsigned int ar5312_WMACsl_IrqGet(WMACsl_id_t *id);
extern void ar5312_WMACsl_UnReset(WMACsl_id_t *id);
extern void ar5312_WMACsl_Enable(WMACsl_id_t *id);
extern WMACsl_id_t * ar5312_WMACsl_init(unsigned int wmac_idx);
extern int ar5312_WMACsl_UnitIsValid(unsigned int wmac_idx);
extern void ar5312_WMACsl_SOCRevInfoGet(unsigned int *SOCVersion,
		                        unsigned int *SOCRevision);

#endif /* __ASM_ATHEROS_WMAC_SUPPORT_H */
