#!/bin/sh
#
# Create a no_dynamic_loader_init.c file that adds the qdiscs and filters
# in the modules that are being compiled to qdisc_list or filter_list,
# respectively. We could just transform the object names directly, except that
# q_fifo.o contains bfifo and pfifo.

MODULES=$*

OUT=no_dynamic_loader_init.c

CFILES=`echo $MODULES | sed -e 's/\.o/.c/g'`

QPATTERN="^struct qdisc_util \(.*\)_util = {$"

QUTILS=`grep -e "$QPATTERN" $CFILES | \
    cut -d: -f2 | \
    sed -e "s/$QPATTERN/\1_util/g"`

FPATTERN="^struct filter_util \(.*\)_util = {$"

FUTILS=`grep -e "$FPATTERN" $CFILES | \
    cut -d: -f2 | \
    sed -e "s/$FPATTERN/\1_util/g"`

cat > $OUT <<EOF
#include "utils.h"
#include "tc_util.h"

extern void qdisc_list_add(struct qdisc_util *q);
extern void filter_list_add(struct filter_util *q);

EOF

for qutil in $QUTILS; do
    echo "extern struct qdisc_util $qutil;" >> $OUT
done

for futil in $FUTILS; do
    echo "extern struct filter_util $futil;" >> $OUT
done

cat >> $OUT <<EOF

void no_dynamic_loader_init(void)
{
EOF

for qutil in $QUTILS; do
    echo "	qdisc_list_add(&$qutil);" >> $OUT
done

for futil in $FUTILS; do
    echo "	filter_list_add(&$futil);" >> $OUT
done

cat >> $OUT <<EOF
}
EOF

