/*
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>

#include "utils.h"
#include "tc_util.h"

// FIXME: this struct should be imported from the same place as our kernel module
/* the highest 802.1d tag */
#define TC_80211_MAX_TAG 7
struct tc_ieee80211_qopt
{
	unsigned char	tag2queue[TC_80211_MAX_TAG + 1];
};

static void print_usage(void)
{
	fprintf(stderr, "Usage: ... 802.11\n"
				"[1dmap <queue for 1d tag 0> ... <queue for 1d tag 7>]\n"
				"[edca <queue number> <aifs> <cw min> <cw max> <tx op limit>]*\n");
}

static int ieee80211_parse_opt(struct qdisc_util *qu, int argc, char **argv, struct nlmsghdr *n)
{
	/* 1d tag to queue mapping defaults taken from 
	 * the WME spec & testplan */
	struct tc_ieee80211_qopt opt={{ 2, 3, 3, 2, 1, 1, 0, 0 }};

	for( ; argc > 0; argc--, argv++) {
		if (strcmp(*argv, "help") == 0) {
			print_usage();
			return -1;
			
		} else if (strcmp(*argv,"1dmap") == 0) {
			int idx;
			for (idx = 0; idx <= TC_80211_MAX_TAG; idx++) {
				int q;
				argc--; argv++;
				if (argc == 0) {
					fprintf(stderr, "not enough queue numbers following 1dmap\n");
					print_usage();
					return -1;
				}
				if (get_unsigned(&q, *argv, 10)) {
					fprintf(stderr, "bad queue number following 1dmap\n");
					print_usage();
					return -1;
				}
				opt.tag2queue[idx] = q;
			}
			continue;
			
		} else {
			fprintf(stderr, "unknown option \"%s\"\n", *argv);
			print_usage();
			return -1;
		}
	}

	addattr_l(n, 1024, TCA_OPTIONS, &opt, sizeof(opt));
	return 0;
}

static int ieee80211_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
{
	struct tc_ieee80211_qopt *qopt;
	int i;

	if (opt == NULL)
		return 0;

	if (RTA_PAYLOAD(opt)  < sizeof(*qopt))
		return -1;

	qopt = RTA_DATA(opt);
	fprintf(f, "1dtag");
	for (i = 0; i <= TC_80211_MAX_TAG; i++)
		fprintf(f, " %d", qopt->tag2queue[i]);
	
	return 0;
}

static int ieee80211_print_xstats(struct qdisc_util *qu, FILE *f, struct rtattr *xstats)
{
	return 0;
}

struct qdisc_util ieee80211_util = {
	NULL,
	"802.11",
	ieee80211_parse_opt,
	ieee80211_print_opt,
	ieee80211_print_xstats,
};

