//
// de.gyronet.kangaroo.io.ResourceConnection.java
// kangaroo
//
// Created by Marc Kleine-Budde on 04.02.2003
// Copyright (c) 2003 GYRO net GmbH. All rights reserved.
//
// inspired by / copied from:
// from the Kaffe Project
//
// see original Copyright below... 

package de.gyronet.kangaroo.io;

import java.io.DataInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.util.Hashtable;
import javax.microedition.io.Connection;
import javax.microedition.io.Connector;
import javax.microedition.io.InputConnection;
import de.gyronet.kangaroo.io.FileConnection;

public class ResourceConnection implements Protocol, InputConnection {

    private InputStream in;
    private boolean open = false;

    protected int openCounter = 0;

    protected int inOpenCounter = 0;
    protected int maxInOpen = 1;

    public Connection open(String target, Hashtable params, int mode, boolean timeouts) throws IOException {
	String classPath = System.getProperty("java.class.path") + 
	    FileConnection.pathSeparator + 
	    System.getProperty("sun.boot.class.path");
	String path = null;

	target = checkTarget(target);

	if (mode != Connector.READ) {
	    throw new IOException("illegal mode: " + mode);
	}

	if (classPath.length() == 0) {
	    throw new IOException("cannot load resource " + target + ", invalid classpath " + classPath);
	}

	int index = classPath.indexOf(FileConnection.pathSeparator);
	if (index > 0) {
	    do {
		path = classPath.substring(0, index);
		classPath = classPath.substring(index + 1);
		index = classPath.indexOf(FileConnection.pathSeparator);

		in = openResource(path, target);
		if (in != null) {
		    open = true;
		    openCounter++;
		    return(this);
		}
	    } while (index > 0);
	}

	in = openResource(classPath, target);
	if (in != null) {
	    openCounter++;
	    open = true;
	    return(this);
	}

	throw new IOException("resource " + target + "not found");
    }

    private String checkTarget(String target) throws IOException {
	if (target.charAt(0) == '.' && target.charAt(1) == '.') {
	    throw new IOException("Illegal characters in target " + target);
	}	    

	int index = target.indexOf(FileConnection.separator + ".." + FileConnection.separator);

	if (index != -1) {
	    throw new IOException("Illegal characters in target " + target);
	}
	
	return target;
    }

    private void closeCon() throws IOException {
	    if (--openCounter == 0) {
		in = null;
	    }
    }

    public void close() throws IOException {
	if (open == true) {
	    open = false;

	    closeCon();
	}
    }

    public InputStream openInputStream() throws IOException {
	ensureOpen();

        return(in);
    }
    
    public DataInputStream openDataInputStream() throws IOException {
	ensureOpen();

        return(new DataInputStream(in));
    }


    private InputStream openResource(String path, String resource) {
	String target;

	target = "file:" + path + FileConnection.separator + resource;
	try {
	    return(Connector.openInputStream(target));
	} catch (IOException e) {
// 	    System.err.println("IOException " + e.getMessage() + " " + target);
	}

	target = "jar:" + path + FileConnection.pathSeparator + resource;
	try {
	    return(Connector.openInputStream(target));
	} catch (IOException e) {
// 	    System.err.println("IOException " + e.getMessage() + " " + target);
	}

	return(null);
    }

    protected void ensureOpen() throws IOException {
        if (!open) {
            throw new IOException("Connection closed");
        }
    }
}
