LINUX_SOURCE=$(OAP_TOPDIR)/apps/atheros/linux
LINUX_SRC=$(LINUX_SOURCE)

EXTRA_CPU_FLAGS=-mcpu=r4600 -mips2 -mabi=32

USR5453=$(OAP_TOPDIR)/config/usr5453

usr5453_tools:
	for tool in toolchain; do \
		(cd apps/$$tool && ./oap_build $(OAP_TOPDIR)/.config build); \
	done

FIRST_TARGET_TOOLS = usr5453_tools

CLEAN_TARGET_TOOLS=

usr5453-kernel:
	make -C $(OAP_TOPDIR)/apps/atheros/linux zImage

IMAGE_TARGETS-$(CONFIG_OAP_FLASHBOOT)	+= usr5453-devices cramfs.img
IMAGE_TARGETS-$(CONFIG_OAP_JTAGBOOT)	+= initrd
IMAGE_TARGETS-y				+= usr5453-kernel image-install
# This must be last
IMAGE_TARGETS-$(CONFIG_OAP_FLASHBOOT)   += upgrade.tar

IMAGE_TARGETS = $(IMAGE_TARGETS-y)

usr5453-device-dirs:
	mkdir -p $(DESTDIR)/dev

usr5453-devices: usr5453-device-dirs
	# This rule only used for CRAMFS creation - the metafile needs existing
	# files to modify the attributes of
	for device in `awk '$$1 == "Device:" {print $$2}' $(USR5453)/dev.cramfs`; do \
		touch $(DESTDIR)/dev/$$device; \
	done

image-install:
	cp $(OAP_TOPDIR)/apps/atheros/linux/arch/mips/boot/zImage \
		$(OAP_TOPDIR)/zImage
	# Temporary image for the JTAG device to download
	cp $(OAP_TOPDIR)/apps/atheros/linux/arch/mips/boot/compressed/vmlinux \
		$(OAP_TOPDIR)/zImage.elf

menuconfig:
	$(MAKE) -C scripts/lxdialog all HOSTCC=$(HOSTCC)
	CONFIG_OAP_PRODUCT_$(PRODUCT)=y $(CONFIG_SHELL) scripts/Menuconfig $(TARGET_CONFIG)
	ln -sf $(OAP_TOPDIR)/.config $(LINUX_SOURCE)
	ln -sf $(OAP_TOPDIR)/config/autoconf.h $(LINUX_SOURCE)/include/linux
	rm -f $(OAP_TOPDIR)/apps/usr5453/mips-linux/sys-include/linux/autoconf.h
	ln -sf $(OAP_TOPDIR)/config/autoconf.h $(OAP_TOPDIR)/apps/usr5453/mips-linux/sys-include/linux
	rm -f $(OAP_TOPDIR)/zImage \
	      $(OAP_TOPDIR)/cramfs.img \
	      $(OAP_TOPDIR)/initrd.gz \
	      $(OAP_TOPDIR)/upgrade.tar
	$(MAKE) apps_config
 
oldconfig: 
	CONFIG_OAP_PRODUCT_$(PRODUCT)=y $(CONFIG_SHELL) scripts/Configure -d $(TARGET_CONFIG)
	ln -sf $(OAP_TOPDIR)/.config $(LINUX_SOURCE)
	ln -sf $(OAP_TOPDIR)/config/autoconf.h $(LINUX_SOURCE)/include/linux
	rm -f $(OAP_TOPDIR)/apps/usr5453/mips-linux/sys-include/linux/autoconf.h
	mkdir -p  $(OAP_TOPDIR)/apps/usr5453/mips-linux/sys-include/linux
	ln -sf $(OAP_TOPDIR)/config/autoconf.h $(OAP_TOPDIR)/apps/usr5453/mips-linux/sys-include/linux
	rm -f $(OAP_TOPDIR)/zImage \
	      $(OAP_TOPDIR)/cramfs.img \
	      $(OAP_TOPDIR)/initrd.gz \
	      $(OAP_TOPDIR)/upgrade.tar
	$(MAKE) apps_config

PATCHES=modversions.patch scr1515.patch

usr5453_setup:
	@echo "No additional steps for setting up USR5453"

define run-genext2fs
	mkdir -p $(DESTDIR)/etc
	date > $(DESTDIR)/etc/buildtime
	MD5SUM=`echo USR5453 | md5sum | cut -d" " -f1` ;\
		echo "$$MD5SUM  -" > $(DESTDIR)/etc/board.md5
	INITRD_SIZE=`du -bs $(DESTDIR) | cut -f 1` ; \
        echo "Size: $$INITRD_SIZE" ; \
	apps/genext2fs/genext2fs \
	    -b `expr $$INITRD_SIZE / 1024 + $(EXTRA_SPACE)` \
	    -v -d $(DESTDIR) -f $(USR5453)/dev.genext2fs $@
endef

define run-mkcramfs
	mkdir -p $(DESTDIR)/etc
	date > $(DESTDIR)/etc/buildtime
	MD5SUM=`echo USR5453 | md5sum | cut -d" " -f1` ;\
		echo "$$MD5SUM  -" > $(DESTDIR)/etc/board.md5
	cp $(USR5453)/dev.cramfs $(DESTDIR)/dev/METAFILE
	INITRD_SIZE=`du -bs $(DESTDIR) | cut -f 1` ; \
	echo "Size: $$INITRD_SIZE" ; \
	apps/cramfs/mkcramfs -m METAFILE $(DESTDIR) $@
endef

define reduce
	$(INSTALL) -d  $(DESTDIR)/lib
	$(INSTALL) -d  $(DESTDIR)/usr/lib
	if [ "$(CONFIG_OAP_DEBUG)" = "y" ] ; then \
		cp -a $(CONFIG_OAP_TOOLCHAIN_DESTDIR)/lib/*so* $(DESTDIR)/lib; \
	else \
		config/usr5453/mklibs -v \
		  --no-default-lib \
		  -L $(CONFIG_OAP_TOOLCHAIN_DESTDIR)/lib \
		  --target=`echo $(CONFIG_OAP_CROSS_COMPILE) | sed -e 's/-$$//'` \
		  -d $(DESTDIR)/lib \
		  --ldlib ld-uClibc.so.0 \
		  `find $(DESTDIR) -type f -perm +0111` ; \
	fi
endef

rootfs-initrd:
	install $(USR5453)/linuxrc $(DESTDIR)/linuxrc
	install $(USR5453)/flash-program $(DESTDIR)/sbin

$(USR5453)/dev.genext2fs:  $(USR5453)/dev.cramfs
	# Generate a genext2fs format metafile for device nodes
	awk '$$1 == "Device:" {print $$3 "rw-	" $$4 "," $$5 "	/dev/" $$2}' \
		$(USR5453)/dev.cramfs > $@

initrd: EXTRA_SPACE=512
initrd: rootfs-initrd $(USR5453)/dev.genext2fs usr5453-device-dirs
	$(reduce)
	$(run-genext2fs)
	gzip -f initrd
	cp initrd.gz $(LINUX_SOURCE)/arch/mips/ramdisk/ramdisk.gz
	$(OAP_IMAGE_INSTALL_FINAL)

cramfs.img: rootfs-initrd
	$(reduce)
	$(run-mkcramfs)
	$(OAP_IMAGE_INSTALL_FINAL)

upgrade.tar: zImage cramfs.img
	ln -sf zImage kernel.bin
	ln -sf cramfs.img rootfs.bin
	MD5SUM=`cat kernel.bin | md5sum | cut -d" " -f1` ;\
		echo "$$MD5SUM  -" > kernel.md5
	MD5SUM=`cat rootfs.bin | md5sum | cut -d" " -f1` ;\
		echo "$$MD5SUM  -" > rootfs.md5
	echo "USR5453" > board
	@# Must use -h here to get the real files not the links
	tar -chf upgrade.tar board kernel.bin kernel.md5 rootfs.bin rootfs.md5
	rm board kernel.bin kernel.md5 rootfs.bin rootfs.md5
