/*
 * Created on Aug 30, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.Activation;

import java.io.IOException;

import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.clustermembership.InvocationFailureException;
import com.instant802.cldc.net.Socket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.TrivialRecordFramer;
import com.instant802.cldc.util.ControlledQueue;
import com.instant802.cldc.util.ControlledThread;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.QueueClosedException;
import com.instant802.cldc.util.ThreadCounter;

class ConnectionProcessor extends Thread implements ControlledThread {
	private static final String TYPE = "ConnectionProcessor";
	private final Activator activator;
	private ControlledQueue incomingConnections;

	public void run() {
		boolean running = true;
		ActivatorRequest request;
		while (running) {
			Socket connection;
			try {
				request = (ActivatorRequest) incomingConnections.dequeue();
				try {
					if (Log.DEBUG) Log.info("dequeued an incoming connection for processing");
					processConnection(request.getConnection(), request.getMessage());
				} finally {
					// connection.close();
				}
			} catch (QueueClosedException e) {
				/* this means that the queue was deliberately closed, so we should shut down */
				running = false;
			} catch (RuntimeException e ) {
				Log.emergency("ConnectionProcessor", e );
			}
		}
	}
	
	private void debug(String string) {
		if (Log.DEBUG) Log.debug("ConnectionProcessor", string );
	}

	void processConnection(Socket connection, InvocationMessage message) {
		try {
			final TrivialRecordFramer framer =
				new TrivialRecordFramer(connection);
			if (Log.DEBUG) info("invoking method");
			final InvocationResult result = activator.invoke(message);
			if (Log.DEBUG) info("got result of activation");
			if (Log.DEBUG) debug( result.asXMLDocument() );
			framer.sendRecord(result.asByteArray());
			if (Log.DEBUG) info("sent result of activation");
		} catch (SocketException e) {
			warning( e );
		} finally {
			connection.close();
		}
	}
	

	private void info(String string) {
		Log.info(string );
	}

	private void warning(Exception e) {
		Log.warning( TYPE, e );
	}
	
	/**
	 * @param incomingConnections
	 */
	public ConnectionProcessor(
		Activator activator,
		ControlledQueue incomingConnections) {
		this.incomingConnections = incomingConnections;
		this.activator = activator;
		ThreadCounter.register( this );
	}

	public void finalize() {
		ThreadCounter.deregister( this );
	}

	/* (non-Javadoc)
	 * @see com.instant802.cldc.util.ControlledThread#waitForStop()
	 */
	public void waitForStop() {
		if( incomingConnections.isOpen() ) {
			incomingConnections.close();
		}
		try {
			this.join();
		} catch (InterruptedException e) {
			warning( e );
		}
	}
}