/*
 * Created on Aug 30, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.Activation;

import com.instant802.cldc.linux.NetInterface;
import com.instant802.cldc.net.ServerSocket;
import com.instant802.cldc.net.Socket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.SocketTimeoutException;
import com.instant802.cldc.net.TrivialRecordFramer;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.ControlledQueue;
import com.instant802.cldc.util.ControlledThread;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.QueueClosedException;
import com.instant802.cldc.util.ThreadCounter;


class ConnectionReceiver extends Thread implements ControlledThread {
	/**
	 * @param incomingConnections
	 */
	private static final String TYPE = "ConnectionReceiver";
	private static final int TIMEOUT = 15000;
	private OrchestratorNode node;
	
	ConnectionReceiver(ControlledQueue incomingConnections, OrchestratorNode node) throws  SocketException {
		this.incomingConnections = incomingConnections;
		createSocket( node.adminInterface() ); 
		/* socket is created here so that an exception generated trying to bind it can be caught and dealt with
		 * before the thread is started.
		 */
		this.node = node;
		ThreadCounter.register( this );
	}
	
	public void finalize() {
		ThreadCounter.deregister( this );
	}
	
	private void createSocket( NetInterface iface ) throws SocketException {
		serverSocket = new ServerSocket();
		serverSocket.bindToInterface( iface );
		serverSocket.listen(Activator.SERVICE_PORT);
		serverSocket.setTimeout( TIMEOUT );
	}
	
	private final ControlledQueue incomingConnections;
	private ServerSocket serverSocket;

	public void run() {
		try {
			while ( incomingConnections.isOpen() ) {
				Socket connection;
				try {
					final TrivialRecordFramer framer;
					connection = (Socket) serverSocket.accept();
					try {
						framer = new TrivialRecordFramer(connection);		
						final byte[] received = framer.receiveRecord();	
						final InvocationMessage message;
						message = new InvocationMessage(received);	
						Service service = node.service(message.getService());
						if (service.isThreadable()) {
							if (Log.DEBUG) Log.debug("Spawning activation: " + new String(received));
							new Thread(new ConnectionHandler(node.activator(), connection, message)).start();
						} else {
							if (Log.DEBUG) Log.info("Qing activation: " + new String(received));							
							incomingConnections.enqueue(new ActivatorRequest(connection, message));
						}
					} catch (QueueClosedException e) {
						connection.close();
					}
				} catch (SocketTimeoutException e ) {
					// these happen continuously so we don't want to report them.in 
				} catch (SocketException e) {
					/* this means that the server socket generated an error
					 * should we shut down, or restart?
					 * TODO: decide this issue
					 * this could be a timeout.
					 */
					 warning( e );
				} catch( Exception e ) {
					Log.emergency("ConnectionReceiver", e );
				}
			}
		} finally {
			closeServerSocket();
		}
	}

	private void closeServerSocket() {
		if (serverSocket != null) {
			serverSocket.close();
		}
	}

	/**
	 * @param e
	 */
	private void warning(Exception e) {
		Log.warning( TYPE, e );
	}

	public void waitForStop() {		
		try {
			if( incomingConnections.isOpen() ) {
				incomingConnections.close();
			}
			this.join();
		} catch (InterruptedException e) {
			warning( e );
		} finally { 
			closeServerSocket(); 
		}
	}
}