/*
 * Created on Aug 22, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.DataPath;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Vector;

import org.kxml2.io.KXmlSerializer;
import org.kxml2.kdom.Element;
import org.kxml2.kdom.Node;
import org.xmlpull.v1.XmlSerializer;

import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class XMLRecord extends GenericRecord {

	private static final String NAME_ATTRIBUTE = "name";
	private static final String NAMESPACE = "";

	/**
	 * 
	 */
	protected XMLRecord() {
	}

	private String typeName, nameAttribute;

	/**
	 * return the typename for this record, which is taken from the XML
	 * element used to construct it.
	 * @see com.instant802.cldc.DataPath.GenericRecord#typeName()
	 */
	public String typeName() {
		return typeName;
	}

	/**
	 * Construct an XML record from an XML Element.  The name of the 
	 * record is taken from the name of the element.  Each child of the element
	 * is assumed to be an element which contains text.  The children are
	 * converted into fields, the element name is the name of the field, and
	 * the element text is the value.
	 * 
	 * @param element The element to retrieve data from.
	 */
	public XMLRecord(Element element) throws XMLRepresentationException {
		createFields(element);
		nameAttribute = element.getAttributeValue(NAMESPACE, NAME_ATTRIBUTE);
	}

	protected void createFields(Element element)
		throws XMLRepresentationException {
		this.typeName = element.getName();
		int children = element.getChildCount();
		for (int i = 0; i < children; i++) {
			if (element.getType(i) == Node.ELEMENT) {
				Element child = element.getElement(i);
				createField(child);
			}
		}
	}

	/**
	 * Given an element which represents a text field,
	 * create a field containing the text.
	 * @fieldElement the element containing the text.
	 */
	protected void createField(Element fieldElement)
		throws XMLRepresentationException {
			
		String text = extractText( fieldElement );
		field( fieldElement.getName(),  text);
	}
	
	/**
	 * Extract all of the text from the children of a given element. AlanL -- modified it to
	 * regard an ELEMENT as text to be serialized. This gives us (very creaky)support for nested
	 * XML datastructures in the Activator (required by some of DK's UI pages). THis is, of course, 
	 * precisely the wrong way to go about it. 
	 * @param fieldElement the element whose text should be returned
	 * @return a string concatenation of all the text children of fieldElement
	 */
	private final String extractText( Element fieldElement )
	{
		StringBuffer text = new StringBuffer();
		for( int i = 0; i < fieldElement.getChildCount(); i ++ ) {
			if( fieldElement.getType( i ) == Node.TEXT ) {
				text.append( fieldElement.getText( i ) );
			} else if (fieldElement.getType(i) == Node.ELEMENT) {
				XmlSerializer serializer = new KXmlSerializer();
				ByteArrayOutputStream os = new ByteArrayOutputStream( );
				try{
					serializer.setOutput( os, getEncoding() );
					Element el;
					for (int count = fieldElement.getChildCount(), j = 0; j < count; ++j) {
						fieldElement.getElement(j).write( serializer );
					}
					serializer.flush();
					os.flush();
				} catch ( IOException e ) {
					Log.critical( "XMLRecord", e );
				}			
				text.append(os.toString());
			}
		}
		return text.toString();
	}

	public XMLRecord(byte[] data) throws XMLRepresentationException {
		Element root = getRootElement(data);
		createFields(root);
		nameAttribute = root.getAttributeValue(NAMESPACE, NAME_ATTRIBUTE);
	}

	public String getNameAttribute() {
		return nameAttribute;
	}

	public Vector keys()
	{
		return keys;
	}
}
