/*
 * Created on Aug 13, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.SessionMonitor;

import com.instant802.cldc.radius.session.AuthenticationHistory;
import com.instant802.cldc.radius.session.AuthenticationRecord;
import com.instant802.cldc.DataPath.GenericRecord;
import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.clustermembership.InvocationFailureException;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class SessionStatusRecord extends GenericRecord {
	
	private static final String AUTH_USERNAME = "auth_username";
	private static final String AUTH_EVENT = "auth_event";
	private static final String AUTH_EVENT_DETAIL = "auth_event_detail";
	private static final String TX_RETRY_FAILED = "tx_retry_failed";
	private static final String TX_PACKETS = "tx_packets";
	private static final String TX_BYTES = "tx_bytes";
	private static final String RX_BYTES = "rx_bytes";
//	private static final String AGE_SEC = "age_sec";
	private static final String ERROR_RATE = "error_rate";
	private static final String TX_AVERAGE = "tx_average";
	private static final String RX_AVERAGE = "rx_average";
	private static final String LOCATION = "location";
	/**
	 * @param assocRecord
	 * @param authHistory
	 */	
	public SessionStatusRecord(OrchestratorNode node, AssociationRecord assocRecord, AuthenticationHistory authHistory) {
		mergeIn( assocRecord );

		if (Log.DEBUG) debug("fetching authentication record");		
		AuthenticationRecord authRecord = authHistory.recordForMac( assocRecord.staMac() );
		if( null != authRecord ) {
			if (Log.DEBUG) debug("got authentication record");
			field( AUTH_USERNAME , authRecord.userName );
			field( AUTH_EVENT , authRecord.authenticationEvent );
			field( AUTH_EVENT_DETAIL, authRecord.eventDetail );
		} else {
			if (Log.DEBUG) debug("no authentication record was available");
		}
		calculateSynthetics();
		try {
			field( LOCATION, node.configurationModel().getOrchestratorConfig().location() );
		} catch (ElementNotFoundException e) {
			// it's not fatal if we can't read the location
			error( e );
		}
	}
	
	private void error(ElementNotFoundException e) {
		Log.error("SessionStatusRecord", e );
	}

	private void calculateSynthetics() {
		calculateErrorRate();
		mapAverages();
		// because we don't have access to the age information, these two data
		// are not calculated.
//		calculateTxAve();
//		calculateRxAve();
	}

	/**
	 * @deprecated this should be removed as soon as the UI utilizes the correct field names.
	 */
	private void mapAverages() {
		intField( TX_AVERAGE, intField( TX_BYTES ) );
		intField( RX_AVERAGE, intField( RX_BYTES ) );
	}
//
//	private void calculateTxAve() {
//		int txBytes = intField( TX_BYTES );
//		int ageSec = intField( AGE_SEC );
//		int txAverage = ageSec > 0 ? (txBytes / 1000) / ageSec : 0;
//		intField( TX_AVERAGE, txAverage );
//	}
//
//	private void calculateRxAve() {
//		int rxBytes = intField( RX_BYTES );
//		int ageSec = intField( AGE_SEC );
//		int rxAverage = ageSec > 0 ? (rxBytes / 1000) / ageSec : 0;
//		intField( RX_AVERAGE, rxAverage );
//	}

	private void calculateErrorRate() {
		int txRetryFailed = intField( TX_RETRY_FAILED );
		int txPackets = intField( TX_PACKETS );
		int errorRate = (txPackets > 0) ? (100 * txRetryFailed) / txPackets : 0;
		intField( ERROR_RATE, errorRate );
	}

	public SessionStatusRecord( XMLRecord record ) throws InvocationFailureException, XMLRepresentationException {	
		makeFromXMLRecord(record);
	}

	public String typeName() {
		return "SessionStatusRecord";
	}

	public String staMac() {
		if (Log.DEBUG) debug("returning station mac "+ field( AssociationRecord.STA_MAC ));
		return field( AssociationRecord.STA_MAC );
	}
	
	public int age() {
		//TODO: deal with the case where this is not set - because an exception is likely to be thrown.
		return Integer.parseInt((String) data.get( AssociationRecord.INACTIVE_MSEC ));
	}
}
