/*
 * Created on Aug 13, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.SessionMonitor;

import java.util.Enumeration;
import java.util.Hashtable;
import com.instant802.cldc.clustermembership.InvocationFailureException;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * Responsible for accumulating status records, maintaining only the most
 * recent one for each client MAC address.
 */
public class SessionStatusTable {
	private final Hashtable table = new Hashtable();
	/**
	 * Add all of the relevant records from the supplied monitor
	 * to the table.
	 * @param monitor
	 */
	public void addRecords(LocalSessionMonitor monitor ) {
		Enumeration e;
		try {
			if (Log.DEBUG) debug("getting records from monitor");
			e = monitor.getCurrentRecords();
			while( e.hasMoreElements() ) {
				SessionStatusRecord record = (SessionStatusRecord) e.nextElement();
				if (Log.DEBUG) debug("adding record");
				addRecord( record );
			}
		} catch (InvocationFailureException e1) {
			if (Log.DEBUG) Log.warning("SessionStatusTable", e1 );
		}
		if (Log.DEBUG) debug("done adding records from monitor");
 	}

	/* add the supplied record to the table if it is relevant
	 */
	private synchronized void addRecord(SessionStatusRecord record) {
		if( mostRecentForMac( record ) ) {
			if (Log.DEBUG) debug("record is most recent so it is accepted");
			table.put( record.staMac(), record );		
		} else {
			if (Log.DEBUG) debug("record has been superceded so is dropped");
		}
	}
	
	private void debug( String message ) {
		Log.debug( "SessionStatusTable", message );
	}
	
	public Enumeration records() {
		return table.elements();	
	}
	
	/* return true if the supplied record is a more recent record for this mac address
	 * than any previously encountered.
	 */
	private boolean mostRecentForMac( SessionStatusRecord record ) {
		if (Log.DEBUG) debug( "checking record age of " +record );
		if (Log.DEBUG) debug("table is "+table );
		SessionStatusRecord existing = (SessionStatusRecord) table.get( record.staMac() );
		if (Log.DEBUG) debug( "found existing record "+ existing );
		if( null == existing ) return true;
		if( record.age() < existing.age() ) {
			if (Log.DEBUG) debug("new record is younger than existing record");
			return true;
		} else {
			if (Log.DEBUG) debug("new record is older than existing record");
			return false;
		}
	}
}

