/*
 * Created on Aug 23, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.TestFramework;

import java.util.Enumeration;
import java.util.Vector;

import com.instant802.cldc.util.SyslogMessage;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public abstract class TestCase extends TestBase {

	boolean hasRun = false;
	protected Vector tests = new Vector();
	protected Vector passed = new Vector();
	protected Vector failed = new Vector();
	protected abstract String getName();

	public void summary( boolean full ) {
		System.out.println("Testing: " + getName()+ " passed: " + getPassed() + "/" + getTotal()  );			
		if( (getFailed() > 0) || full ) {
			if( getPassed() > 0 ) {
				System.out.println("   PASSED: "); 
				summarylist( passed );
			}
			if( getFailed() > 0 ) {
				System.out.println("   FAILED: ");
				summarylist( failed );
			}
		}
	}

	/**
	 * @param passed
	 */
	private void summarylist(Vector list) {
		Enumeration messages = list.elements();
		while( messages.hasMoreElements()) {
			System.out.println("      " + (String) messages.nextElement());
		}
	}

	/**
	 * called to test the validity of an assertion. 
	 * @param condition the condition under test
	 * @param string an english description of the condition under test.
	 */
	protected synchronized void assertTrue(boolean condition, String message) {
		if( condition ) {
			passed( message );
		} else {
			failed( message );
		}
	}

	/**
	 * called to indicate the failure of a given test which is described by the message.
	 * @param message
	 */
	private void failed(String message) {
		failed.addElement( message );
		logResult( "FAILED: " + message );
	}

	/**
	 * write a message to the output log
	 * @param string the message to write.
	 */
	private void logResult(String string) {
		System.out.println( string );
	}

	/**
	 * called to indicate the success of a given test which is described by the message.
	 * @param message
	 */
	private void passed(String message) {
		passed.addElement( message );
		logResult( "PASSED: " + message );
	}

	
	/**
	 * class used to wrap individual test methods to control the 
	 * setup and teardown of the test fixture.
	 * @author robin
	 *
	 */
	protected abstract class Test {
		protected abstract void test() throws Exception;
		void run() {
			try{
				setUp();
				test();
				tearDown();
			} catch ( Exception e ) {
				info( e );
			}
		}
	}
	
	/**
	 * This method contains code that will be run before each test method.
	 */
	protected abstract void setUp();
	
	/**
	 * This method contains code that will be run after each test method.
	 */
	protected abstract void tearDown();
	
	/**
	 * add a test to the set defined by the concrete class.
	 * @param test the test method (encapsulated in a 'Test' class) to add.
	 */
	protected void add( Test test ) {
		tests.addElement( test );
	}
	
	/**
	 * defines the tests to be run by the concrete class using the 'add'
	 * method to add 'Test' objects which wrap test methods.
	 */
	protected abstract void suite();

	/**
	 * The method which should be called to run all of the tests defined in this class.
	 */	
	public synchronized void run() {
		SyslogMessage.setProcessname( "testcase "+getName() );
		if( ! hasRun ) {		
			hasRun = true;
			suite();
			Enumeration e = tests.elements();
			while( e.hasMoreElements() ) {
				Test t = (Test) e.nextElement();
				info(" ... NEXT TEST METHOD ...");
				t.run();
			}
		} 
	}

	/**
	 * @return
	 */
	public int getTotal() {
		return getPassed() + getFailed();
	}

	/**
	 * @return
	 */
	public int getPassed() {
		return passed.size();
	}

	/**
	 * @return
	 */
	public int getFailed() {
		return failed.size();
	}
}