/*
 * Created on Jul 15, 2004
 * 
 * To change the template for this generated file go to Window - Preferences -
 * Java - Code Generation - Code and Comments
 */
package com.instant802.cldc.channelplanner;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.LSQLClient;
import com.instant802.cldc.LSQL.LSQLResultSet;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.san.AP;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.SimpleCommaDelimParser;
import com.instant802.cldc.util.SimpleDelimParser;

/**
 * @author alan
 * 
 * To change the template for this generated type comment go to Window -
 * Preferences - Java - Code Generation - Code and Comments
 */
class ChannelPlanRecord extends XMLRecord
{

	private static final String TYPE = "ChannelPlanRecord";
	public static final String INTERFERENCE_RATIO = "interference_ratio";
	public static final String TS = "time";
	public static final String ENERGY_THRESHOLD = "energy_threshold";
	public static final String WLAN_THRESHOLD = "wlan_threshold";
	public static final String GATHER_INTERVAL = "gather_interval";
	public static final String PLAN_INTERVAL = "plan_interval";
	public static final String CURRENT_BAG = "current_bag";
	public static final String LOCKED_APS = "locked_aps";
	public static final String MODE = "mode";
	public static final String MSG = "msg";
	
	private String current_bag;
	private int channelbag[];
	private Vector locked_aps;

	public ChannelPlanRecord(byte[] bs) throws XMLRepresentationException
	{
		super(bs);
		updateBags();
		updateLockedAPs();
	}

	private ChannelPlanRecord()
	{
	}

	private void updateLockedAPs()
	{
		String svar;
		locked_aps = new Vector();

		if (null != (svar = stringField("locked_aps")))
		{
			// Due to the idiotic semantics of apconf we have to have
			if (!svar.trim().equals("none"))
			{
				SimpleCommaDelimParser scp = new SimpleCommaDelimParser(svar);
				String field;
				while (null != (field = scp.nextField()))
				{
					locked_aps.addElement(field.trim());
				}
			}
		}
	}

	public Vector getLockedAps()
	{
		return locked_aps;
	}

	public boolean isLocked(String ip) {
		Enumeration locked = locked_aps.elements();
		while (locked.hasMoreElements()) {
			if (ip.equals((String) locked.nextElement())) {
				return true;
			}
		}
		return false;
	}

	public String typeName()
	{
		return TYPE;
	}
	
	public void saveState(OrchestratorNode node) throws Exception
	{
		// For now do this pedantically ...
		LSQLResultSet lquery;
		StringBuffer sb = new StringBuffer();
		String op;

		lquery =
			new LSQLClient(node.schema()).query(
				"<select><channel-planner/></select>");
		if (lquery.counter() == 1)
		{ // update
			op = "update>";
		} else if (lquery.counter() == 0)
		{
			op = "insert>";
		} else
		{
			Log.error("Mangled config ? (> 1 ChannelPlanner node ?)");
			return;
		}
		sb = sb.append("<" + op + "<channel-planner>");
		enTag(sb, ENERGY_THRESHOLD, stringField(ENERGY_THRESHOLD));
		enTag(sb, WLAN_THRESHOLD, stringField(WLAN_THRESHOLD));
		enTag(sb, GATHER_INTERVAL, stringField(GATHER_INTERVAL));
		enTag(sb, PLAN_INTERVAL, stringField(PLAN_INTERVAL));
		enTag(sb, CURRENT_BAG, stringField(CURRENT_BAG));
		enTag(sb, LOCKED_APS, stringField(LOCKED_APS));
		enTag(sb, MODE, stringField(MODE));

		sb.append("</channel-planner></" + op);
		lquery = new LSQLClient(node.schema()).query(sb.toString());

		if (1 != lquery.counter())
			Log.error("LSQL query failed to update ChannelPlanner");
		else
			new LSQLClient(node.schema()).setRunningConfig();
	}

	public String toString()
	{
		return "ChannelPlannerConfig: " + asXMLDocument();
	}

	public String getMode()
	{
		return stringField(MODE);
	}

	public void setMode(String mode) 
	{
		field(MODE, mode);
	}
	
	public int getEnergyThreshold()
	{
		String i = stringField(ENERGY_THRESHOLD);
		if (null == i)
			return 50;
		else
			return Integer.parseInt(i);
	}

	public int getWLANThreshold()
	{
		String i = stringField(WLAN_THRESHOLD);
		if (null == i)
			return 1000;
		else
			return Integer.parseInt(i);
	}

	// Field value is in minutes - we require ms
	public long getPlanInterval()
	{
		String i = stringField(PLAN_INTERVAL);
		if (null == i)
			return 60 * 1000L;
		else
			return 60 * 1000L * Long.parseLong(i);
	}

	private void updateBags()
	{
		String svar;
		current_bag = "<none>";
		if (null != (svar = stringField("bags")))
		{
			SimpleBarParser sbp = new SimpleBarParser(svar);
			SimpleCommaDelimParser scp;
			String rec, channel, name;
			Hashtable channel_bags = new Hashtable();
			String band;

			name = null;
			while (null != (rec = sbp.nextField()))
			{
				Vector channels = new Vector();
				scp = new SimpleCommaDelimParser(rec);
				if (null != (name = scp.nextField().trim()))
				{
					if (null != (band = scp.nextField()))
					{
						band = band.trim().toLowerCase();
						channels.addElement(band);
						while (null != (channel = scp.nextField()))
						{
							channels.addElement(
								new Integer(Integer.parseInt(channel.trim())));
						}
						channel_bags.put(name, channels);
						continue;
					}
				}
				if (Log.DEBUG)
					debug("Bad channel bags information in channel_planner");
				break;
			}
			if (null == rec)
			{ // Correct termination of bags processing
				if (null != (name = stringField("current_bag").trim()))
				{
					Vector channels = (Vector) channel_bags.get(name);
					if (null != channels)
					{
						channelbag = new int[channels.size() - 1];
						band =
							((String) channels.elementAt(0))
								.trim()
								.toLowerCase();
						for (int i = 0; i < channelbag.length; ++i)
						{
							channelbag[i] =
								((Integer) channels.elementAt(i + 1))
									.intValue();
						}
						current_bag = name;
					} else
					{
						if (Log.DEBUG)
							debug("current_bag not in bags");
					}

				} else
				{
					if (Log.DEBUG)
						debug("current_bag not defined");
				}
			}
		}
	}

	public int[] getChannelBag()
	{
		return channelbag;
	}

	public String getCurrentBagName()
	{
		return current_bag;
	}

	public Vector getLockedAPs()
	{
		return locked_aps;
	}

	public void updateRecord(XMLRecord record)
	{
		Enumeration keys = record.asHashTable().keys();
		while (keys.hasMoreElements())
		{
			String key = (String) keys.nextElement();
			String value = (String) record.asHashTable().get(key);
			field(key, value);	
		}
		updateBags();
		updateLockedAPs();
	}

	/**
	 * @return
	 */
	public String currentBand()
	{
		return AP.getBand(channelbag);
	}
}

class SimpleBarParser extends SimpleDelimParser
{
	public SimpleBarParser(String toParse)
	{
		super(toParse, '|');
	}
}
