/*
 * Created on Aug 13, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.clustermembership;

import java.util.Enumeration;
import java.util.Hashtable;

import com.instant802.cldc.Activation.Activator;
import com.instant802.cldc.Activation.InvocationMessage;
import com.instant802.cldc.Activation.InvocationResult;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.net.Socket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.TrivialRecordFramer;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * Representation of a cluster member that can be used to perform network
 * operations.
 * 
 * @author robin
 */
public abstract class ClusterMember {

	private static final int TIMEOUT = 15000;

	private void debug(String m) {
		Log.debug("ClusterMember", m);
	}

	public InvocationResult InvokeMethod(String service, String method)
	throws InvocationFailureException {
		return InvokeMethod(service, method, null);
	}
	
	/**
	 * Invoke a named method on a named service provided by this member.
	 * 
	 * @author robin
	 * @throws InvocationFailureException
	 *             thrown if any part of the communication fails
	 */
	public InvocationResult InvokeMethod(String service, String method, Hashtable args)
		throws InvocationFailureException {
		if (Log.DEBUG) debug("trying to invoke " + service + "." + method);
		
		final Socket socket;
		
		try {
			socket = new Socket();
		} catch (SocketException e) {
			throw new InvocationFailureException( e.toString() );
		}

		try {
			socket.setTimeout(TIMEOUT);
			try {
				if (Log.DEBUG) debug("connecting socket");
				socket.connect(ipAddress(), Activator.SERVICE_PORT);
				if (Log.DEBUG) debug("socket connected");
			} catch (SocketException e) {
				throw new InvocationFailureException(
					"could not connect to activator at " + ipAddress());
			}

			// Send the message.
			InvocationMessage message =
				new InvocationMessage(service, method);
			if (null != args) {
				// Args should be String/String pairs
				Enumeration keys = args.keys();
				while (keys.hasMoreElements()) {
					String key = keys.nextElement().toString();
					String value = args.get(key).toString();
					message.field(key, value);
				}
			}
			final TrivialRecordFramer framer = new TrivialRecordFramer(socket);
			try {
				if (Log.DEBUG) debug("sending message");
				sendMessage(framer, message);
				if (Log.DEBUG) debug("message sent");
			} catch (SocketException e1) {
				throw new InvocationFailureException(
					"failed sending message to activator: " + e1.getMessage());
			}

			// Read the response
			byte[] received;
			try {
				if (Log.DEBUG) debug("waiting for response");
				received = framer.receiveRecord();
				if (Log.DEBUG) debug("received response record");
			} catch (Exception e2) {
				throw new InvocationFailureException(
					"failed receiving message from activator: "
						+ e2.getMessage());
			}

			// Construct the result from the response
			InvocationResult result;
			try
			{
				result = new InvocationResult(received);
			} catch (XMLRepresentationException e3)
			{
				throw new InvocationFailureException(e3.toString());
			}
			// now check to see whether result is an exception.
			InvocationFailureException exception = result.getException();
			if (null != exception)
				throw exception;

			if (Log.DEBUG) debug("returning result " + result);
			return result;
		} finally {
			socket.close();
		}
	}

	/**
	 * send an invocation message through a socket connected to the member
	 * 
	 * @param socket
	 *            a socket connected to the member
	 * @param message
	 *            the message to send
	 */
	private void sendMessage(
		TrivialRecordFramer framer,
		InvocationMessage message)
		throws SocketException {
		StringBuffer xml = new StringBuffer();
		message.appendAsXML(xml);
//		byte buffer[] = xml.toString().getBytes();
		// ALan's Non-standard optimization -- see StringBuffer in Kaffe
		byte buffer[] = xml.getBytes();
		framer.sendRecord(buffer);
	}

	/**
	 * @return the current IP address of the member.
	 */
	public abstract String ipAddress();

	/**
	 * @return true if the member is the local system.
	 */
	public abstract boolean isNode(OrchestratorNode node);

	public abstract String id();

	public abstract String location();
	
	public abstract String radioMacs(); 

}
