/*
 * Created on Sep 7, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.clustermembership;

import java.io.IOException;
import java.util.Enumeration;

import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.channelplanner.WirelessNeighborhoodRecords;
import com.instant802.cldc.clustermastering.AgentNotRunningException;
import com.instant802.cldc.clustermastering.InvalidTimingException;
import com.instant802.cldc.deviceconfiguration.ClusteredAP;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.deviceconfiguration.OrchestratorConfig;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * this class is responsible for seeking out a cluster, and joining it, or otherwise causing a new cluster to be formed.
 */
public class MembershipAgent implements ClusterMembership {

	public static final String NON_MEMBER = "non_member";
	private final OrchestratorNode node;
	private final MembershipList members;
	
	public MembershipAgent( OrchestratorNode node ) {
		this.node = node;
		members = new MembershipList( node.masterCriteria(), node.configurationModel() );
	}

	public synchronized Enumeration getMembers() {
		//TODO: return actual members list
		return members.elements();
	}
	
	public synchronized boolean isMember() throws ElementNotFoundException {
		return ! node.masterCriteria().getClusterID().equals( NON_MEMBER );
	}

	private synchronized void startCluster() throws SocketException, ElementNotFoundException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException {
		if (Log.DEBUG) Log.notice("starting cluster locally");
		setClusterID( node.getNodeId() );
	//	agent.clusterStarted()
		if (Log.DEBUG) Log.info("set cluster id, now accepting self to cluster");
		acceptSelf();
		notifyAll();
		if (Log.DEBUG) Log.info("done starting cluster locally");
	}

	/**
	 * @param string
	 */
	private void setClusterID(String string) throws LSQLException, APConfException, XMLRepresentationException, IOException, MalformedHTTPMessageException, SocketException {
		OrchestratorConfig config = node.configurationModel().getOrchestratorConfig();
		config.setClusterID( string );
		node.configurationModel().writeToAPConf();
	}

	private final void acceptSelf() throws ElementNotFoundException, LSQLException, APConfException, XMLRepresentationException, IOException, MalformedHTTPMessageException, SocketException {
		acceptMember( new MembershipRequestMessage( node ));
	}

	public synchronized void handleMembershipRequest(MembershipRequestMessage message) throws SocketException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException {
		if( canJoin( message )) {
			approveMembership( message );
		} else {
			if (Log.DEBUG) Log.notice("denying membership to node  "+message.getSenderId() );
			message.sendReply( node, new MembershipDeniedMessage( node ) );
		}
	}

	private final boolean canJoin( MembershipRequestMessage message ) {
		if( memberIsNew( message ) ) { 	
			if( clusterIsFull() ) {
				return false;
			} else {
				return acceptingNewMemberships();
			}
		} else {
			//member is already in list
			return true;
		}
	}
	
	private final boolean memberIsNew(MembershipRequestMessage message) {
		boolean isNew = ! members.containsMember( message );
		return isNew;
	}

	private final void approveMembership(MembershipRequestMessage message)
		throws
			XMLRepresentationException,
			IOException,
			MalformedHTTPMessageException,
			LSQLException,
			APConfException,
			SocketException,
			ElementNotFoundException {
		acceptMember( message );
		message.sendReply( node, new MembershipAcceptedMessage( node ) );
		node.clusterMasterAgent().makeAnnouncement();			
	}

	private final boolean acceptingNewMemberships() {
		try {
			boolean ignore = node.configurationModel().getOrchestratorConfig().ignoreNewAPs();
			return ! ignore;
		} catch (ElementNotFoundException e) {
			error( e );
			// if we can't read the configuration policy, then we don't form clusters.
			return false;
		}
	}

	private void error(ElementNotFoundException e) {
		Log.error( "MembershipAgent", e );
	}

	private synchronized void acceptMember(MembershipRequestMessage message) throws LSQLException, APConfException, XMLRepresentationException, IOException, MalformedHTTPMessageException, SocketException {
		if (Log.DEBUG) Log.notice("accepting new member "+message.getSenderId()+" to cluster \n" );
		members.addMember( new ReceivedClusterMember( message ) );
		// So CHP etc. reflect new APs ASAP -- only if adding subsequent members to the cluster
		
		// ****** There may be an asynchronous deadlock (or at least time-sink)issue here which has
		// ****** croaked clustering. So for now this ie being taken out -- which will mean that
		// ****** clustered wireless nbhd info may take a few minutes (up to 15) to catch
		// ****** up with the cluster configuration itself. This will get reseolved ... alanl
		
//		if (! node.adminInterface().ipAddress().equals(message.getSenderIp()))
//				WirelessNeighborhoodRecords.updateNow();
	}

	private final synchronized boolean clusterIsFull() {
		boolean full = members.size() >= node.getMembershipSizeLimit();
		if (Log.DEBUG) info("cluster is full: "+full );
		return full;
	}

	public synchronized void membershipDenied() {
		if (Log.DEBUG) Log.notice("membership request denied");
		node.enterStandalone();
	}

	public synchronized void membershipAccepted(MembershipAcceptedMessage message) throws LSQLException, APConfException, XMLRepresentationException, IOException, MalformedHTTPMessageException, SocketException {
		if (Log.DEBUG) Log.notice("membership request accepted");
		setClusterID( message.getClusterId() );
		if (Log.DEBUG) Log.debug("cluster id set");
		notifyAll();
		if (Log.DEBUG) Log.debug("done handling membership accepted");
	}

	public synchronized void controlMembership() throws ElementNotFoundException, SocketException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException {
		if (Log.DEBUG) info("taking control of membership");
		if( ! isMember() ) {
			if (Log.DEBUG) info("this node is not already a member, so starting a new cluster");
			// if we are not already a member, then we start a new cluster.
			startCluster();
		} else {
			if (Log.DEBUG) info("this node is already a member, so not starting a new cluster");
			// here we need to check whether the ip address of the existing member is correct or not.
			acceptSelf();
		}
	}

	private void info(String string) {
		if (Log.DEBUG) Log.info( "MembershipAgent", string );
	}

	public int memberCount() {
		return members.size();
	}

	public void configurationSynchronized() throws ElementNotFoundException, InvalidTimingException, SocketException, AgentNotRunningException {
		if (Log.DEBUG) info("handling configuration synchronization");
		if (Log.DEBUG) info("node is "+node );
		if (Log.DEBUG) info("agent is "+node.clusterMasterAgent() );
//		if( node.coordinationAgent().isServer() ) {
		if (Log.DEBUG) info("populating membership list from received configuration");
			members.populate();
//		}
		if (Log.DEBUG) info("done handling synchronization");
	}

	public synchronized void dropClusteredAPs() throws LSQLException, APConfException {
		members.removeAll();
	}

	/**
	 * return the set of aps in the configuration that are currently marked for removal.
	 */
	public synchronized MarkedForRemoval getMarkedForRemoval() {
		Enumeration e = node.configurationModel().getClusteredAPs();
		MarkedForRemoval results = new MarkedForRemoval( members, node );
		while( e.hasMoreElements() ) {
			ClusteredAP ap = (ClusteredAP) e.nextElement();
			if (Log.DEBUG) info("ap "+ap.id()+" marked for removal "+ap.isMarkedForRemoval() );
			if( ap.isMarkedForRemoval() ) {
				results.add( ap );
			}
		}
		return results;
	}
}
