/*
 * Created on Sep 29, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.coordination;

import java.io.IOException;

import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.clustermastering.AgentNotRunningException;
import com.instant802.cldc.clustermastering.ClusterMaster;
import com.instant802.cldc.clustermastering.ClusterMasterAgent;
import com.instant802.cldc.clustermastering.InvalidTimingException;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.messaging.BadMessageException;
import com.instant802.cldc.messaging.Message;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.ControlledThread;
import com.instant802.cldc.util.Log;
import com.instant802.cldc.util.ThreadCounter;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class CoordinationAgent extends Thread implements ControlledThread {
	
	private final int COORDINATOR_RETRY_DELAY= 10000;
	private Coordinator coordinator;
	private boolean running = true;
	private boolean active = true;
	private final OrchestratorNode node;
	private boolean isClient = false;
	
	public CoordinationAgent( OrchestratorNode node ) {
		this.node = node;
		ThreadCounter.register( this );
	}

	public void finalize() {
		ThreadCounter.deregister( this );
	}

	public void run() {
		try{
			active = node.configurationModel().getOrchestratorConfig().clusterable();			
			while( running ) {
				while( active ) {
					node.nodeStatusRecord().setClusterStatus("seeking cluster...");
						try {							
							findCoordinator();
							if (Log.DEBUG) Log.notice("starting coordinator");
							coordinator.coordinate();						
							if (Log.DEBUG) Log.notice("coordination finished normally");
						} catch (IOException e1) {
							error( e1 );
						} catch (XMLRepresentationException e1) {
							error( e1 );
						} catch (MalformedHTTPMessageException e) {
							error( e );
						} catch (LSQLException e) {
							node.handleLSQLException( e );
						} catch (APConfException e) {
							error( e );
						} catch (BadMessageException e ) {
							error( e );
						} catch (InvalidTimingException e) {
							error( e );
						} catch (CoordinatorLostException e) {
							error( e );
						} catch (SocketException e) {
							error( e );
						} catch (Exception e) {
							e.printStackTrace();
						} 	finally {
							try {
								node.lsqlMessageReceiver().clearLocallyModified();
							} catch (LSQLException e3) {
								error( e3 );
							} catch (APConfException e3) {
								error( e3 );
							}	
							synchronized( this ) {
								isClient = false;
								coordinator = null;
							}
							try {
								if (Log.DEBUG) Log.info("now invalidating master");
								masterAgent().invalidateMaster();
							} catch (ElementNotFoundException e2) {
								error( e2 );
							} catch (SocketException e2) {
								error( e2 );
							}
						}
				}
				node.nodeStatusRecord().setClusterStatus("disconnected from cluster");
				if( running ) {
					synchronized( this ) {
//						stopped = true;
						notifyAll();
						awaitReactivation();
//						stopped = false;
					}
				}
			}
//		} catch (MessageNotUnderstoodException e1) {
//			emergency( e1 );
		} catch (RuntimeException e1) {
			emergency(e1);
//		} catch (AgentNotRunningException e) {
//			emergency(e);
		} catch (ElementNotFoundException e) {
			emergency(e);
		} catch (Exception e) {
//			debug("================ " + e.toString());
		} 
	}

	private synchronized void awaitReactivation() {
		while( (! active) && running ) {
			try {
				wait( 1000 );
			} catch (InterruptedException e2) {
				error( e2 );
			}
		}
	}

	private ClusterMasterAgent masterAgent() {
		return node.clusterMasterAgent();
	}

	private void emergency( Exception e1) {
		Log.emergency("CoordinationAgent",e1 );
	}

	private void findCoordinator() throws ElementNotFoundException, AgentNotRunningException {
		ClusterMaster master;
		
		while( null == coordinator ) {
			try {
				master = masterAgent().getMaster();
				if( master.isLocal() ) {
					if (Log.DEBUG) Log.notice("creating coordinator server locally after negotiation");
					createServer();
				} else {
					if (Log.DEBUG) Log.notice("creating coordinator client after negotiation");
					createClient(master);
				}
			} catch (SocketException e) {
				error( e );
				try{
					masterAgent().invalidateMaster();
				} catch ( SocketException f ) {
					// this is legacy
				}
			} catch (InvalidTimingException e) {
				error( e );
			}
			if (Log.DEBUG) Log.info("pausing before next attempt to start coordinator");
			pause( COORDINATOR_RETRY_DELAY );
		}
	}

	private synchronized void createServer() throws SocketException {
		isClient = false;
		coordinator = new CoordinatorServer( node );
	}

	private synchronized void createClient(ClusterMaster master) throws SocketException {
		isClient = true;
		coordinator = new CoordinatorClient( node, master );
	}

	private synchronized void pause( int delay ) {
		try{
			wait( delay );
		} catch( InterruptedException e ) {
			error( e );
		}
	}

	public synchronized void stopCoordinator() {
		if( null != coordinator ) {
			if (Log.DEBUG) info("calling StopCoordinating");
			coordinator.stopCoordinating();
			if (Log.DEBUG) info("returned from StopCoordinating");
		}
	}

	private void info(String string) {
		Log.info("CoordinationAgent", string );
	}

	private void error( Exception e) {
		Log.error("CoordinationAgent", e );
	}

	public void waitForStop() {
		running = false;
		deactivate();
		try {
			if (Log.DEBUG) info("attempting to join coordinationAgent thread");
			joinIfNotCurrentThread( this );
			if (Log.DEBUG) info("returned from joining coordinationAgent thread");
		} catch (InterruptedException e) {
			error( e );
		}
		if (Log.DEBUG) info("coordination agent permanently stopped");
	}

	public synchronized void activate() {
		active = true;
		notifyAll();
	}

	public synchronized void deactivate() {
		if (Log.DEBUG) info("setting active state to false");
		active = false;
		if (Log.DEBUG) info("asking coordinator to stop");
		stopCoordinator();
//		waitUntilStopped();
		if (Log.DEBUG) info("done deactivating coordination agent");
	}

	/**
	 * join the supplied thread unless it is the current thread.
	 */
	private void joinIfNotCurrentThread(Thread toJoin) throws InterruptedException {
		Thread current = Thread.currentThread();
		if( ! current.equals( toJoin ) ) {
			info("thread is not the current thread, so joining");
			toJoin.join();
		} else {
			info("thread to join is the same as the current thread, so not joining");
		}
	}

	public void broadcast( Message message ) throws XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException, SocketException {
		Coordinator touse = coordinator;
		if( touse != null ) {
			if (Log.DEBUG) debug("forwarding message to coordinator");
			touse.broadcast( message );
		} else {
			if (Log.DEBUG) debug("no coordinator yet - so message is dropped");
		}
	}


	private void debug(String string) {
		Log.debug("CoordinationAgent", string );
	}

	public synchronized boolean isServer() {
		if( null == coordinator ) {
			return false;
		}
		return coordinator.isServer();
	}

	public synchronized boolean isActive() {
		if( null != coordinator ) {
			return coordinator.isActive();
		} else {
			return false;
		}
		
	}

	public void handleProbeResponse(ProbeResponseMessage message) {
		Coordinator coordinator = this.coordinator;
		if( null != coordinator ) {
			coordinator.handleProbeResponse( message );
		}
	}

	public void checkForDeletions() 
	{
		Coordinator coordinator = this.coordinator;
		if( null != coordinator ) {
			coordinator.checkForDeletions();
		}
	}

	public boolean isClient() {
		if (Log.DEBUG) debug("returning "+isClient+" for isClient" );
		return isClient;
	}

	public void masterChanged() {
		Coordinator coordinator = this.coordinator;
		if( null != coordinator ) {
			if (Log.DEBUG) debug("master changed and we are running a coordinator - so stopping it");
			coordinator.stopCoordinating();
		} else {
			if (Log.DEBUG) debug("master changed but we are not yet running a coordinator - so ignoring change");
		}
	}
}
