/*
 * Created on Sep 29, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.coordination;

import java.io.IOException;

import com.instant802.cldc.DataPath.XMLRecord;
import com.instant802.cldc.DataPath.XMLRepresentationException;
import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.clustermastering.AgentNotRunningException;
import com.instant802.cldc.clustermastering.ClusterMaster;
import com.instant802.cldc.clustermastering.InvalidTimingException;
import com.instant802.cldc.clustermembership.MarkedForRemoval;
import com.instant802.cldc.clustermembership.MembershipAcceptedMessage;
import com.instant802.cldc.clustermembership.MembershipDeniedMessage;
import com.instant802.cldc.clustermembership.MembershipRequestMessage;
import com.instant802.cldc.deviceconfiguration.ElementNotFoundException;
import com.instant802.cldc.deviceconfiguration.OrchestratorConfig;
import com.instant802.cldc.messaging.BadMessageException;
import com.instant802.cldc.messaging.Message;
import com.instant802.cldc.messaging.MessageClassifier;
import com.instant802.cldc.messaging.MessageNotUnderstoodException;
import com.instant802.cldc.messaging.ReplyMechanism;
import com.instant802.cldc.net.MalformedHTTPMessageException;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.SocketTimeoutException;
import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.synchronization.RequestSynchronizationMessage;
import com.instant802.cldc.synchronization.SynchronizationSetMessage;
import com.instant802.cldc.transfer.TransferBase;
import com.instant802.cldc.util.DateUtil;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class CoordinatorClient extends TransferBase implements Coordinator, ReplyMechanism {

	private static final long PROBE_TIMEOUT = SOCKET_TIMEOUT * 3;
	private static final long LONG_PROBE_TIMEOUT = PROBE_TIMEOUT * 4;
	private static final long JOIN_TIMEOUT = SOCKET_TIMEOUT * 5; // this is long to allow for slow processing.
	private static final String[] MEMBERSHIP_RESPONSES = { MembershipDeniedMessage.TYPE, MembershipAcceptedMessage.TYPE };
	private static final String TYPE = "CoordinatorClient";
	private final OrchestratorNode node;
	private boolean running = true;
	private boolean active = false;
	private long timeout;
	
	public CoordinatorClient( OrchestratorNode node, OrchestratorConfig config) throws SocketException {
		super( node.schema(), node.sslSessionContext() );
		this.node = node;
		connect( config.getCoordinatorAddress(), node.getCoordinationPort() );
	}

	/**
	 * @param master
	 */
	public CoordinatorClient( OrchestratorNode node, ClusterMaster master) throws SocketException {
		super( node.schema(), node.sslSessionContext() );
		this.node = node;
		connect( master.getAddress(), node.getCoordinationPort() );
	}

	public void coordinate() throws ElementNotFoundException, MessageNotUnderstoodException, BadMessageException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException, InvalidTimingException, SocketException, AgentNotRunningException, CoordinatorLostException {
		if( joinCluster() ) {
			Log.notice("joined cluster - trying to synchronize configurations");
			synchronize();
			active = true;
			maintainSynchronization();
		}
	}

	/**
	 * 
	 */
	private boolean joinCluster() throws MessageNotUnderstoodException, BadMessageException, ElementNotFoundException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException, InvalidTimingException, SocketException, AgentNotRunningException, CoordinatorLostException {
		final MembershipRequestMessage message = new MembershipRequestMessage( node );
		setClusterStatus("connecting to cluster...");
		sendMessage( message );
		Message received = null;
		long starttime = DateUtil.now();
		while( null == received ) {
			try{
				received = getNextMessage( MEMBERSHIP_RESPONSES );
			} catch (SocketTimeoutException e ) {
				long duration = (DateUtil.now() - starttime);
				if( duration > JOIN_TIMEOUT ) {
					throw new CoordinatorLostException("coordinator did not respond to membership request after "+duration+" milliseconds.");
				}
			}
		}
		if( received.typeName().equals( MembershipAcceptedMessage.TYPE ) ) {
			received.onReceipt( node );
			return true;
		} else {
			received.onReceipt( node );
			return false;
		}
	}

	Message getNextMessage( String[] acceptable ) throws SocketException, IOException, XMLRepresentationException, MessageNotUnderstoodException, BadMessageException {
		XMLRecord record = receive();
		Message received = MessageClassifier.classifyMessage( this, record );
		for( int i = 0; i < acceptable.length; i ++ ) {
			if( received.typeName().equals( acceptable[ i ] )) {
				return received;
			}
		}
		//TODO: message not understood at this point is not as severe as general - normally it means that there's a protocol difference, whereas here
		//TODO: it could just mean that things are out of sync. - probably a different exception should be thrown.
		throw new MessageNotUnderstoodException( record );
	}

	private void sendMessage( Message message) throws SocketException {
		send( message );
	}

	private void maintainSynchronization() throws MessageNotUnderstoodException, BadMessageException, ElementNotFoundException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException, InvalidTimingException, SocketException, AgentNotRunningException, CoordinatorLostException {
		resetProbeTimout();
		while( running ) {
			setClusterStatus("connected to cluster");
			try {
				Message received = getNextMessage( new String[] {SynchronizationSetMessage.TYPE, ProbeResponseMessage.TYPE } );
				if (Log.DEBUG) Log.debug("Message processed");
				resetProbeTimout();
				received.onReceipt( node );
				resetProbeTimout();
			} catch (SocketTimeoutException e) {
				// NOT AN ERROR;		
				// if we don't receive anything, then we should send a 
				// probe request.
				if (Log.DEBUG) Log.debug ("Handling socket read timeout");
				checkTimout();
				sendMessage( new ProbeRequestMessage( node ) );
			} 
		}
	}

	private void checkTimout() throws CoordinatorLostException {
		long duration = (DateUtil.now() - timeout);
		if( duration > PROBE_TIMEOUT ) {
			throw new CoordinatorLostException("coordinator failed to respond to a probe request ("+
							   (duration - PROBE_TIMEOUT) );
		}
	}

	private void synchronize() throws MessageNotUnderstoodException, BadMessageException, ElementNotFoundException, XMLRepresentationException, IOException, MalformedHTTPMessageException, LSQLException, APConfException, InvalidTimingException, SocketException, AgentNotRunningException {
		Message message = new RequestSynchronizationMessage( node );
		sendMessage( message );
		SynchronizationSetMessage received = (SynchronizationSetMessage) getNextMessage( new String[] {SynchronizationSetMessage.TYPE} );
		received.onReceipt( node );
	}

	/**
	 * check whether this AP has been deleted and act accordingly.
	 */
	public void checkForDeletions() {
		if (Log.DEBUG) Log.info("checking for deletion of this node");
		MarkedForRemoval marked = node.membershipAgent().getMarkedForRemoval();
		if( marked.includesThis() ) {
			setClusterStatus("removed from cluster - going into standalone mode...");
			node.enterStandalone();
		}
	}

	public void stopCoordinating() {
		if (Log.DEBUG) Log.info("handling request to stop coordinating");
		running = false;
	}

	public void sendMessage(Message message, String address) throws IOException, SocketException {
		sendMessage( message );
	}

	public void broadcast(Message msg) {
		if( active ) {
			try {
				if (Log.DEBUG) Log.info("forwarding message to server for broadcast");
				sendMessage( msg );
				setLongProbeTimeout();
			} catch (SocketException e) {
				error( e );
			}
		}
	}

	/**
	 * kludgy way to allow enough time for synchronization requests to be processed.
	 */
	private void setLongProbeTimeout() {
		timeout = DateUtil.now() + LONG_PROBE_TIMEOUT;
	}

	private void error(SocketException e) {
		Log.error( TYPE, e );
	}

	public boolean isServer() {
		return false;
	}

	public boolean isActive() {
		return active;
	}

	public void handleProbeResponse(ProbeResponseMessage message) {
		resetProbeTimout();
	}

	private void resetProbeTimout() {
		timeout = DateUtil.now();
	}

	private void setClusterStatus( String message ) {
		node.nodeStatusRecord().setClusterStatus( message ); 
	}	
}
