/*
 * Created on Nov 4, 2003
 * author: Robin Barooah
 * (C) 2003 Instant802 Networks Inc
 */
package com.instant802.cldc.deviceconfiguration;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.NoSuchElementException;

import com.instant802.cldc.LSQL.APConfException;
import com.instant802.cldc.LSQL.LSQLException;
import com.instant802.cldc.util.Log;

/**
 * @author robin
 */
public class IndexedConfiguration implements ObjectList {

	protected final Hashtable types = new Hashtable();

	public synchronized void addObject(ConfigObject toAdd) {
		Hashtable typeTable = getTypeTable( toAdd.typeName() );
		Object key = toAdd.keyValue();
		typeTable.put( key, toAdd );
	}

	public void debugDump() {
		Enumeration tables = types.elements();
		while( tables.hasMoreElements() ) {
			Hashtable type = (Hashtable) tables.nextElement();
			debugType( type );
		}
	}
	
	private void debugType( Hashtable type) {
		Enumeration types = type.keys();
		while( types.hasMoreElements() ) {
			Object key = types.nextElement();
			ConfigObject obj = (ConfigObject) type.get( key );
			if (Log.DEBUG) debug("key: "+key.toString()+" object: "+obj.asXMLDocument());
		}
	}

	/**
	 * return a hashtable containing objects of the given type within the configuration.
	 */
	protected synchronized Hashtable getTypeTable(String type) {
		Hashtable table = (Hashtable) types.get( type );
		if( null == table ) {
			table = new Hashtable();
			types.put( type, table );
		}
		return table;
	}

	public Enumeration elements() {
		return new IndexEnumeration();
	}

	public synchronized int size() {
		int count = 0;
		Enumeration tables = types.elements();
		while( tables.hasMoreElements() ) {
			Hashtable typeTable = (Hashtable) tables.nextElement();
			count += typeTable.size();
		}
		return count;
	}

	private class IndexEnumeration implements Enumeration {
		
		private final Enumeration tables = types.elements();
		Enumeration typeTable;

		private final void scan() {
			if( typeTable != null ) {
				if( typeTable.hasMoreElements() ) {
					// the next element is available from the typeTable.
					return;
				} else {
					scanTables();
					return;
				}
			}
			scanTables();
			return;
		}

		private final void scanTables() {
			while( tables.hasMoreElements() ) {
				typeTable = ((Hashtable) tables.nextElement()).elements();
				if( typeTable.hasMoreElements() ) {
					// the next type table has elements
					return;
				}
			}
			typeTable = null;
			// there were no elements in any type table
			return;
		}

		public boolean hasMoreElements() {
			scan();
			return null != typeTable;
		}

		public Object nextElement() {
			scan();
			return typeTable.nextElement();
		}
	}

	public synchronized void deleteObject(ConfigObject object) throws LSQLException, APConfException {	
		if (Log.DEBUG) debug("deleting object from model: " + object.typeName());
		Hashtable typeTable = getTypeTable( object.typeName() );
		Object removed = typeTable.remove( object.keyValue() );
		if (Log.DEBUG) debug("removed "+removed );
	}

	protected ConfigObject findDefinition( ConfigObject candidate ) {
		Hashtable typeTable = getTypeTable( candidate.typeName() );
		ConfigObject found = (ConfigObject) typeTable.get( candidate.keyValue() );
		return found;
	}

	private void debug(String string) {
		Log.debug("IndexedConfiguration", string );
	}

	public String toString() {
		StringBuffer out = new StringBuffer();
		appendAsXML( out );
		return out.toString();
	}

	public boolean defines(ConfigObject object) {
		ConfigObject found = findDefinition( object );
		return null != found;
	}

	/**
		 * @param candidate
		 * @return
		 */
	public ConfigObject definition(ConfigObject candidate) {
		ConfigObject found = findDefinition( candidate );
		if( null == found ) {
			throw new NoSuchElementException( "Objectlist does not contain an object of type: "+candidate.typeName()+" with the appropriate key");
		} 
		return found;
	}

	public void appendAsXML(StringBuffer out) {
		Enumeration e = elements();
		while( e.hasMoreElements() ) {
			ConfigObject record = (ConfigObject) e.nextElement();
			record.appendAsXML( out );
		}
	}
}
