/*
 * Created on Jan 16, 2004
 * author: Robin Barooah
 * (C) 2003 Instant802 Networks Inc
 */
package com.instant802.cldc.deviceconfiguration;

import com.instant802.cldc.orchestrator.OrchestratorNode;
import com.instant802.cldc.util.Log;


public class ObjectChangeProcessor {

	private final OrchestratorNode node;
	
	public ObjectChangeProcessor( OrchestratorNode node ) {
		this.node = node;
	}
	
	public void handleObjectChange( ConfigObject changed ) {
		if (Log.DEBUG) info("received changed object to handle "+changed.typeName() );
		if( changed.typeName().equals( InterfaceConfig.TYPE ) ) {
			InterfaceConfig iface = new InterfaceConfig( changed );
			handleInterfaceChanged( iface );
		}
	}

	private void info(String string) {
		if (Log.DEBUG) Log.info("ObjectChangeProcessor", string );
	}

	private void handleInterfaceChanged(InterfaceConfig iface) {		
		String currentAddress = node.adminInterface().ipAddress();
		String newAddress = iface.ipAddress().trim();
		if (Log.DEBUG) Log.info("received updated interface definition for "+iface.getNameAttribute()+"- current address is "+currentAddress+"; new address is "+newAddress );
		Log.debug( "admin interface device name is: "+node.adminInterface().getDeviceName() );
		Log.debug( "received interface definition device name is: "+iface.getNameAttribute() );
		if( node.adminInterface().getDeviceName().equals( iface.getNameAttribute() ) ) {
			Log.debug( "interface names match" );			
			if( ! currentAddress.equals( newAddress )	) {
				if (Log.DEBUG) Log.debug("and addresses have changed");
				if (Log.DEBUG) Log.debug("restarting node");
				try { Thread.sleep(5000L); } catch (Exception _) {}
				node.restart();
			}
		}
		Log.debug("updated interface definition did not cause a restart");
	}
}

