/*
 * Created on Jun 8, 2004
 * author: Robin Barooah
 * Copyright (C) 2004 Instant802 Networks, Inc. All Rights Reserved.
 */
package com.instant802.cldc.hostapd;

import com.instant802.cldc.DataPath.GenericRecord;
import com.instant802.cldc.util.SimpleTsvParser;
import com.instant802.cldc.radius.Utils;

/**
 * @author robin
 */
public class NeighboringAP extends GenericRecord {

	private  static final String MAC_ADDRESS = "mac_address";
	private  static final String BEACON_INTERVAL = "beacon_interval";
	private  static final String CAPABILITY = "capability";
	private  static final String SSID = "ssid";
	private  static final String WPA = "wpa";
	private  static final String PHY_TYPE = "phy_type";
	private  static final String CHANNEL = "channel";
	private  static final String RATE = "rate";
	private  static final String SIGNAL = "signal";
	private  static final String ERP = "erp";
	private  static final String RX_BEACON_COUNT = "rx_beacon_count";
	private  static final String LAST_BEACON_TIMESTAMP = "last_beacon_timestamp";
	private  static final String SUPPORTED_RATES = "supported_rates";
	public static final String BAND_B_G = "b/g";
	public static final String BAND_A = "a";
	public static final String BAND_UNKNOWN = "unknown";
		
	
	private static final String TYPE = "NeighboringAP";
	
	protected NeighboringAP() {}
	/**
	 * Construct a Neighboring AP record from a string containing a tab separated
	 * list of strings. See hostapd api documentation for the command AP-NEXT for
	 * more details on this format.
	 */
	public NeighboringAP(String reply) {
		SimpleTsvParser parser = new SimpleTsvParser( reply );
		// we can get away with this because the parser will return null after the end of the defined fields
		// and the Generic Record will translate these into UNDEFINED_FIELD automatically.
		field( MAC_ADDRESS, parser.nextField() );
		field(BEACON_INTERVAL, parser.nextField());
		field( CAPABILITY,  parser.nextField() );
		field( SSID,  Utils.plainHex(parser.nextField().getBytes()));
		field( WPA,  parser.nextField() );
		field( PHY_TYPE,  parser.nextField() );
		field( CHANNEL,  parser.nextField() );
		field( RATE,  parser.nextField() );
		field( SIGNAL,  parser.nextField() );
		field( ERP,  parser.nextField() );
		field( RX_BEACON_COUNT,  parser.nextField() );
		field( LAST_BEACON_TIMESTAMP,  parser.nextField() );
		field( SUPPORTED_RATES,  parser.nextField() );
	}
	
	public String typeName() {
		return TYPE;
	}

	/**
	 * return the mac address for the AP described by this record.
	 */
	public String macAddress() {
		return field( MAC_ADDRESS );
	}
	
	public int channel()
	{
		return  Integer.parseInt(field(CHANNEL));
	}	
	
	public int signal() {
		return Integer.parseInt(field(SIGNAL));
	}
	
	public String lastBeaconTime() {
		return field(LAST_BEACON_TIMESTAMP);
	}
	
	public int beaconInterval() {
		return Integer.parseInt(field(BEACON_INTERVAL));
	}
	
	public long lastBeaconTS(){
		return Long.parseLong(field(LAST_BEACON_TIMESTAMP));
	}
	
	public int rate() {
		return Integer.parseInt(field(RATE));		
	}
	
	public String ssid() {
		return field(SSID);
	}
	
	public String band() 
	{
		// Look up band from phy_type:
		//	4,7 => "b/g"
		// 8, 256 => a
		// rest -> unknown
		int type = Integer.parseInt(field(PHY_TYPE));
		if (type == 4 || type == 7)
			return BAND_B_G;
		else if (type == 8 || type == 256)
			return BAND_A;
		else
			return BAND_UNKNOWN;
	}
}
