/*
 * Created on Sep 18, 2003
 * 
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.instant802.cldc.net;

import com.instant802.cldc.util.Log;

/**
 * @author robin, alanl
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class UDPDatagramSocket extends SocketBase implements DatagramSocket
{
	public UDPDatagramSocket() throws SocketException
	{
		this(0);
	}
	
	public UDPDatagramSocket(int port) throws SocketException
	{
		constructDatagram();
		try{
			int err = nativeBind(sd, port);
			if (err != 0) {
				close(); // if we don't close here we potentially leak an SD
				throw new SocketException("couldn't bind datagram socket to port "+port);
			}
		} catch ( NullPointerException e ) {
			exitOnNullPointer( e );
		}
		
	}
	
	/**
	 * @param string
	 */
	private void debug(String string)
	{
		Log.debug("DatagramSocket", string);
	}
	
	public int sendTo(byte[] d, String address, int port)
	throws SocketException
	{
		try{
			int sent = 0;
			if( d.length < 1 ) {
				throw new SocketException("cannot send a datagram of size "+d.length );
			}
			try {
				checkOpen();			
			} catch (SocketException e)	{
				throw new SocketException(
						"datagram socket no longer open: "
						+ address
						+ " port: "
						+ port
						+ " errno is: "
						+ -sent);
			}
			
			sent = nativeSendTo(sd, address, port, d, d.length);
			
			if (sent < 0)	{
				throw new SocketException(
						"couldn't send datagram to address: "
						+ address
						+ " port: "
						+ port
						+ " errno is: "
						+ -sent);
			}
			return sent;
		} catch ( NullPointerException e ) {
			exitOnNullPointer( e );
			return 0;
		}
		
	}
	
	/**
	 * The return 64 bit quantity is as follows: If the bottom 16 bits threated
	 * as a signed 16 bit quantity are > 0 then The top 32 bits are IP address
	 * of sender The next 16 bits are port of sender The bottom 16 bits are
	 * (signed0 length of data Else if bttom 16 bits == 0 then no data was
	 * ready Else an error Alanl
	 */
	private long receiveFrom(byte[] buffer, int buffersize) throws SocketException
	{
		try{
			long res = 0;
			short read = -1;
			int err;
			
			while (read < 0)
			{
				checkOpen();
				res = nativeReceiveFrom(sd, buffer, buffersize);
				
				read = (short) res;
				err = (int) -(res >> 32);
				
				if (read < 0) {
					throw new SocketException("error during read in receiveFrom:" + err);
				}
			}
			if (0 == read)
				debug("NULL datagram ");
			return res;
		} catch ( NullPointerException e ) {
			exitOnNullPointer( e );
			return 0l;
		}
		
	}
	
	public void enableBroadcast() throws SocketException
	{
		checkOpen();
		int err = nativeSetBroadcast(sd);
		if (err != 0) {
			throw new SocketException("couldn't set socket into broadcast mode");
		}
	}
	
	/**
	 * Returns the datagram read by the socket. Note that the data is _copied_
	 * into the returned datagram - buf is used simply for temporary storage
	 * which, for efficiencey reasons, is declared elsewhere
	 */
	public Datagram receive(byte[] buf) throws SocketTimeoutException, SocketException
	{
		try{
			long res = receiveFrom(buf, buf.length);
			Datagram p = new Datagram(buf, (short) res);
			int ip = (int) ((res >> 32) & 0xFFFFFFFF);
			p.setPort((int) ((res >> 16) & 0x0000FFFF));
			p.setAddress(ip2str(ip));
			debug(
					"Received datagram from ip ="
					+ p.getAddress()
					+ ":"
					+ p.getPort()
					+ " size = "
					+ (res & 0xFF));
			return p;
		} catch ( NullPointerException e ) {
			exitOnNullPointer( e );
			return null;
		}
		
	}
	
	public void send(Datagram p) throws SocketException
	{
		sendTo(p.getData(), p.getAddress(), p.getPort());
	}
}
