/*
 * Created on Apr 17, 2003
 * 
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;
import com.instant802.cldc.radius.session.*;
import com.instant802.cldc.ssl.*;
import com.instant802.cldc.util.Log;

/**
 * @author alan
 * 
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class EAPHandler extends RadiusRequestHandler
{
	/*
	 * (non-Javadoc)
	 * 
	 * @see radius.RequestHandler#handled(radius.RadiusPacket)
	 */
	public static final int ATTR_EAP_MESSAGE = 79;

	public static final int ATTR_MSG_AUTHENTICATOR = 80;

		public static final int EAP_CODE_INVALID = 0, // internal
	EAP_CODE_REQUEST = 1,
		EAP_CODE_RESPONSE = 2,
		EAP_CODE_SUCCESS = 3,
		EAP_CODE_FAILURE = 4;

	public static final int EAP_TYPE_IDENTITY = 1,
		EAP_TYPE_NOTIFICATION = 2,
		EAP_TYPE_NAK = 3,
		EAP_TYPE_MD5 = 4,
		EAP_TYPE_OTP = 5,
		EAP_TYPE_GTC = 6,
		EAP_TYPE_PEAP = 25,
		EAP_TYPE_MS_EAP = 26,
		EAP_TYPE_MSCHAPV2 = 29,
		EAP_TYPE_EXTENSIONS = 33;

	//	private static final int SSL_CONTEXT_MODE_SERVER = 1,
	//		SSL_CONTEXT_MODE_CLIENT = 2,
	//		SSL_CONTEXT_MODE_PEER = 3;

	//	private static SSLSessionContext sslcontext =
	//		new SSLSessionContext(SSL_CONTEXT_MODE_SERVER);
	//
	//	private static SSL ssl = new SSL(sslcontext);

	EAPSessionTable eapsessionstate;

	private AuthenticationHistoryImpl authhist;

	public EAPHandler(RadiusServer server)
	{
		super(server);
		eapsessionstate = server.getEAPSessionState();
		this.authhist =
			(AuthenticationHistoryImpl) server
				.getProxy()
				.getAuthenticationHistory();
	}

	public boolean handled(RadiusPacket rp, byte[] secret)
	{
		boolean iseap;
		byte[] eapmsg;
		byte[] msgauthenticator;
		// Handle fragmentation
		eapmsg = rp.getAttributeConcatValues(ATTR_EAP_MESSAGE);
		if (null != eapmsg)
		{
			if (null
				!= (msgauthenticator =
					rp.getAttributeValue((ATTR_MSG_AUTHENTICATOR))))
			{
				if (!Utils
					.compareBytes(
						msgauthenticator,
						rp.computeRequestAuthenticator(secret)))
				{
					return true; // Failed authentication
				}
			}
			EAPPacket eap = new EAPPacket(eapmsg);
			switch (eap.getCode())
			{
				case EAP_CODE_RESPONSE :
					if (Log.DEBUG)
						Log.info(
							">> Processing EAP. Radius ID = "
								+ rp.getIdentifier()
								+ " EAP ID = "
								+ eap.getId());
					doResponse(eap, rp, secret);
					break;
				default :
					if (Log.DEBUG)
						Log.info("Unexpected EAP code: " + eap.getCode());
			}
		}
		return (null != eapmsg);
	}
	/**
	 * Simple state machine for our use of EAP. Currently only support MD5 or
	 * PEAP so return with an PEAP Start. If we get Nak'd then try a Chap
	 * challenge. in response to EAP-IDENTITY ACCESS_REQUEST.
	 * 
	 * @param eap
	 */
	private void doResponse(EAPPacket eap, RadiusPacket rp, byte[] secret)
	{
		byte[] user = rp.getUser();
		EAPState eapstate = null;
		int id = rp.getIdentifier();
		RadiusResponse resp = new RadiusResponse(id);
		int eapcode = EAPHandler.EAP_CODE_INVALID;

		eapstate = eapsessionstate.getEAPState(rp);
		if (null != eapstate)
		{ // We must be etsbalishing IDENTITY  so we expect a paricular EAPID
			if (!eapstate.checkEapRequestId(eap.getId()))
			{
				if (Log.DEBUG)
					Log.debug(
						">> Dropping out of sequence EAP packet. Expecting id = "
							+ eapstate.getEapRequestId());
				return;
			}
		}
		switch (eap.getType())
		{
			case EAP_TYPE_IDENTITY :
				// Generate and embed EAPState info in radius packet
				if (Log.DEBUG)
					Log.debug(">> EAP_TYPE_IDENTITY");
				if (null == rp.getState())
				{
					eapstate = eapsessionstate.createEAPState();
					eapstate.setState(EAPState.EAP_STATE_PEAP_REQUEST_PHASE1);
					// For SessionTracking
					eapstate.setUsermac(rp.getUserMac());
					eapstate.setUsername(new String(rp.getUser()).trim());
					//
					rp.setState(eapstate.getId());
					new EAPPacket(
						EAP_CODE_REQUEST,
						EAP_TYPE_PEAP,
						(byte) (eap.getId() + 1),
						PEAPPacket.startPacket()).embedInRadiusResponse(
						resp);
					eapcode = EAP_CODE_REQUEST;
				} else
				{
					if (Log.DEBUG)
						Log.debug("Got EAP state in IDENTITY");
					eapcode = EAP_CODE_FAILURE;
				}
				break;

			case EAP_TYPE_NOTIFICATION :
				if (Log.DEBUG)
					Log.debug(">> EAP_TYPE_NOTIFICATION");
				break;

			case EAP_TYPE_NAK :
				if (Log.DEBUG)
					Log.debug(">> EAP_TYPE_NAK");
				eapcode = EAP_CODE_FAILURE;
				break;

			case EAP_TYPE_PEAP :
				if (Log.DEBUG)
					Log.debug(">> EAP_TYPE_PEAP");
				eapstate = eapsessionstate.getEAPState(rp);
				if (null != eapstate)
					// Part of a full session and not some random packet
				{
					eapcode =
						new PEAPMethod(server, user, secret).doPeapResponse(
							eap,
							eapstate,
							resp,
							rp.getRequestAuthenticator());
				} else
				{
					if (Log.DEBUG)
						Log.debug(
							"Unexpected PEAP packet (not part of EAP session)");
					eapcode = EAP_CODE_FAILURE;
				}
				break;

			default :
				// Look up from dynamic table here ..

				if (Log.DEBUG)
					Log.info("EAP Type not supported: " + eap.getCode());
				eapstate = eapsessionstate.getEAPState(rp);
				if (null != eapstate)
				{
					eapstate.setEventdetail(EAPState.ESED_BAD_EAP_TYPE);
				}
				break;
		}

		int radiuscode = 0;

		switch (eapcode)
		{
			case EAP_CODE_FAILURE :
				if (Log.DEBUG)
					Log.info("<< EAP_CODE_FAILURE");
				radiuscode = RadiusPacket.R_REJECT;
				authhist.createRecord(radiuscode, eapstate);
				if (null != eapstate)
				{
					eapstate.drop();
				}
				eapstate = null;
				break;

			case EAP_CODE_REQUEST :
				EAPPacket eaprequest = (EAPPacket) resp.getAssociatedpacket();
				if ((null != eaprequest) && (null != eapstate))
					eapstate.setEapRequestId(eaprequest.getId());
				if (Log.DEBUG)
					Log.info("<< EAP_CODE_REQUEST");
				radiuscode = RadiusPacket.R_CHALLENGE;
				authhist.createRecord(radiuscode, eapstate);
				break;

			case EAP_CODE_RESPONSE :
				if (Log.DEBUG)
					Log.info("<< EAP_CODE_RESPONSE");
				radiuscode = RadiusPacket.R_REQUEST;
				break;

			case EAP_CODE_SUCCESS :
				if (Log.DEBUG)
					Log.info("<< EAP_CODE_SUCCESS");
				radiuscode = RadiusPacket.R_ACCEPT;
				authhist.createRecord(radiuscode, eapstate);
				if (null != eapstate)
				{
					eapstate.drop();
				}
				eapstate = null;
				break;

			default :
				if (Log.DEBUG)
					Log.info("<< Invalid EAP_CODE in Radius Response");
				radiuscode = RadiusPacket.R_REJECT;
				authhist.createRecord(radiuscode, eapstate);
				if (null != eapstate)
				{
					eapstate.drop();
				}
				eapstate = null;
				break;
		}
		resp.setCode(radiuscode);

		if (null != eapstate)
		{
			if (Log.DEBUG)
				Log.info("Adding RADIUS_ATTR_STATE to response");
			resp.addAttribute(RadiusPacket.RADIUS_ATTR_STATE, eapstate.getId());
		}
		if (Log.DEBUG)
			Log.info("Sending EAP response to server");
		server.respond(resp.createResponsePacket(rp, secret, true));
	}
}
