package com.instant802.cldc.radius;

/**
 * @author alan
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class EAPPacket
{
	byte[] data;


	public EAPPacket(byte[] data)
	{
		this.data = data;
	}

	/**
	 * @param inbuf
	 * @param bytesread
	 */
	public EAPPacket(byte[] inbuf, int bytesread)
	{
		byte[] data = new byte[bytesread];
		System.arraycopy(inbuf, 0, data, 0, bytesread);
		this.data = data;
	}
	
	public EAPPacket(int code, int type, byte id)
	{
		this(code, type, id, null, 0, 0);
	}

	public EAPPacket(int code, int type, byte id, byte[] data)
	{
		this(code, type, id, data, 0, data.length);
	}

	public EAPPacket(int code, int type, byte id, byte[] payload, int len)
	{
		this(code, type, id, payload, 0, len);
	}

	public EAPPacket(int code, int type, byte id, byte[] payload, int offset, int len)
	{
		this.data = new byte[len + 5];
		setCode(code);
		setType(type);
		setId(id);
		setLength(len + 5);
		if (null != payload)
			System.arraycopy(payload, offset, this.data, 5, len);
	}

	public EAPPacket(int code, byte id)
	{
		data = new byte[4];
		setCode(code);
		setId(id);
		setLength(4);
	}

	public static EAPPacket failurePacket(byte id) {
		return new EAPPacket(EAPHandler.EAP_CODE_FAILURE, id);
	}
	
	public byte getCode()
	{
		return data[0];
	}

	public void setCode(int code)
	{
		data[0] = (byte) (0xFF & code);
	}

	public void setType(int type)
	{
		data[4] = (byte) (0xFF & type);
	}

	public byte getId()
	{
		return data[1];
	}

	public void setId(byte id)
	{
		data[1] = id;
	}

	public int getLength()
	{
		return (256 * (0xff & data[2])) + (0xff & data[3]);
	}

	public void setLength(int length)
	{
		data[2] = (byte) (0xff & ((0xff00 & length) >> 8));
		data[3] = (byte) (0xff & length);
	}

	/**
	 * @return
	 */
	public int getType()
	{
		return data[4];
	}

	public byte[] getData() {
		byte[] buf = new byte[getLength() - 4];
		System.arraycopy(data, 4, buf, 0, getLength() - 4);
		return buf;		
	}
	
	public byte[] getTypeData()
	{
		byte[] buf = new byte[getLength() - 5];
		System.arraycopy(data, 5, buf, 0, getLength() - 5);
		return buf;
	}
	
	public int getDataOffset() {
		return 4;
	}
	
	public int getTypeDataOffset() {
		return 5;
	}
	
	public int getDataLength() {
		return getLength() - 4;
	}
	
	public int getTypeDataLength() {
		return getLength() - 5;
	}

	/**
	 * Embed this EAP packet in the Radius Response. We may have to fragment this EAP message
	 * as mutliple EAP attributes. See PEAP RFC [RFC2869] section 5.13
	 * @param resp
	 */
	public void embedInRadiusResponse(RadiusResponse resp)
	{
		int len = getLength();
		int attrcount = 1 + (len / 254);
		int attrlen, offset;
		byte[] attrbuf;

		for (int i = 0; i < attrcount; ++i)
		{
			offset = i * 253;
			attrlen = (len - offset <= 253) ? len - offset : 253;
			attrbuf = new byte[attrlen];
			System.arraycopy(data, offset, attrbuf, 0, attrlen);
			resp.addAttribute(EAPHandler.ATTR_EAP_MESSAGE, attrbuf);
		}
		resp.setAssociatedpacket(this);
	}
}
