/*
 * Created on Jun 14, 2003
 * 
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.instant802.cldc.radius;

import com.instant802.cldc.ssl.SSLMem;
import java.util.Hashtable;

/**
 * @author alan
 * 
 * See TODO in PEAPMethod -- the various bits of (PEAP) tunneled protocol state
 * here should be moved to specific handler objects and just the objects kept
 * here.
 *  
 */
public class EAPState
{
	public static final int /* EAP State */
	EAP_STATE_PEAP_REQUEST_PHASE1 = 1, /*
																					   * Staring
																					   * PEAP
																					   * TLS
																					   * negotiation
																					   */
	EAP_STATE_PEAP_REQUEST_PHASE2 = 2, /*
																					   * Finished
																					   * PEAP
																					   * TLS
																					   * negotiation
																					   */
	EAP_STATE_PEAP_REQUEST_MSCHAPV2 = 3, /*
																							  * Proposed
																							  * MSCHAPV2
																							  */
	EAP_STATE_PEAP_REQUEST_MD5 = 4, /*
																			    * Proposed
																			    * MD5
																			    * (testing)
																			    */
	/*
	 * The PEAP/MSChap protocol is not clear (or at least I was confused).
	 * There seem to be 3 reply phases: 1. ACK or NACK it in a tunneled CHAP
	 * packet using MSCHapV2 success/failure (with embedded strings - see RFC
	 * 2759). 2. Then ACK or NACK it using tunneled MS_EAP_EXTENSIONS (see MS
	 * PEAP 0 spec) 3. Then ACK or NACK it using EAP-SUCCESS or EAP-FAIL clear
	 * text packets
	 */
	EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_1 = 5,
	/* MSChapV2 success - replied with MSChap Packet */
	EAP_STATE_PEAP_PASSED_AUTH_MSCHAP_2 = 6,
	/* MSChapV2 success - replied with Chap Packet */
	EAP_STATE_PEAP_FAILED_AUTH_MSCHAP_1 = 7,
	/* MSChapV2 failure - replied with MSChap Packet */
	EAP_STATE_PEAP_FAILED_AUTH_MSCHAP_2 = 8,
	/* MSChapV2 failure - replied with Chap Packet */
	EAP_STATE_EAP_MD5_START = 10;

	/*
	 * This is really authentication detail state and as such should be higher
	 * up. But for now all our authentiation details are kept in eapstate. When
	 * we do a reasonable and overdue refactoring then this should probably
	 * migrate up[ to the radius level.
	 */
	public static final int /* Event details */
	ESED_NONE = 0,
		ESED_AUTHENTICATED = 1,
		ESED_BAD_CREDENTIALS = 2,
		ESED_REAUTHENTICATED = 3,
		ESED_BAD_EAP_TYPE = 4,
		ESED_PEAP_PROTOCOL_FAILURE = 5,
		ESED_MSCHAPV2 = 6,
		ESED_INTERNAL_ERROR = 7;
	/* Zounds! A bug ! Check log. */

	public static final String /* Event details in string form */
	ESEDS_NONE = "No more information",
		ESEDS_AUTHENTICATED = "Succesful authentication",
		ESEDS_BAD_CREDENTIALS = "Bad credentials supplied",
		ESEDS_REAUTHENTICATED = "Succesful reauthentication",
		ESEDS_BAD_EAP_TYPE = "Bad EAP type in protocol",
		ESEDS_PEAP_PROTOCOL_FAILURE = "PEAP protocol failure",
		ESEDS_MSCHAPV2 = "MSChapV2",
		ESEDS_INTERNAL_ERROR = "Internal error";

	private SSLMem sslfilter = null;

	private int state, eap_request_id;

	private byte[] id, mpperecvkey, mppesendkey;

	private Hashtable sessiontable;

	private String username, usermac; // Supplicant
																 // for this
																 // session

	private int eventdetail;

	private long ts;

	private EAPState()
	{

	}

	public EAPState(byte[] id, Hashtable sessiontable)
	{
		this.id = id;
		this.sessiontable = sessiontable;
		mpperecvkey = null;
		mppesendkey = null;
		eventdetail = ESED_NONE;
		ts = System.currentTimeMillis();
		eap_request_id = -1;
	}

	public void setEapRequestId(int id) {
		eap_request_id = 0xff & id;
	}
	
	public boolean checkEapRequestId(byte id) {
		if (eap_request_id == -1) // Not expecting a particular id (i.e. pre IDENTITY)
			return true;
		return ((byte) (0xff & eap_request_id) == id);
	}
	
	public int getEapRequestId() {
		return eap_request_id;
	}
	
	public void setState(int state)
	{
		this.state = state;
	}

	public int getState()
	{
		return this.state;
	}

	/**
	 * @return
	 */
	public SSLMem getSslfilter()
	{
		return sslfilter;
	}

	/**
	 * @param mem
	 */
	public void setSslfilter(SSLMem mem)
	{
		sslfilter = mem;
	}

	public byte[] getId()
	{
		return id;
	}

	/**
	 * Remove this entry from the EAP session table. Free any (non-java)
	 * resources associated with it. Allow regular java gc to pick up the entry
	 * (it may currently be accessed and in use if we are dropping because of
	 * timeout - this is inconvenient and inefficient but we drop it anyway and
	 * let retry cure all ills).
	 */
	public void drop()
	{
		sessiontable.remove(id);
		if (null != sslfilter) {
			sslfilter.close();
		}
		sslfilter = null;
	}

	/**
	 * Is this session stale ?
	 * 
	 * @return true if authentication session is over 100 seconds old
	 */
	public boolean isStale()
	{
		return ts - System.currentTimeMillis() > 100000;
	}

	/**
	 * @return
	 */
	public byte[] getMpperecvkey()
	{
		return mpperecvkey;
	}

	/**
	 * @return
	 */
	public byte[] getMppesendkey()
	{
		return mppesendkey;
	}

	/**
	 * @return
	 */
	public Hashtable getSessiontable()
	{
		return sessiontable;
	}

	/**
	 * @param bs
	 */
	public void setMpperecvkey(byte[] bs)
	{
		mpperecvkey = bs;
	}

	/**
	 * @param bs
	 */
	public void setMppesendkey(byte[] bs)
	{
		mppesendkey = bs;
	}

	/**
	 * @param hashtable
	 */
	public void setSessiontable(Hashtable hashtable)
	{
		sessiontable = hashtable;
	}

	/**
	 * @return
	 */
	public String getUsermac()
	{
		return usermac;
	}

	/**
	 * @return
	 */
	public String getUsername()
	{
		return username;
	}

	/**
	 * @param string
	 */
	public void setUsermac(String string)
	{
		usermac = string;
	}

	/**
	 * @param string
	 */
	public void setUsername(String string)
	{
		username = string;
	}

	/**
	 * @return
	 */
	public int getEventdetail()
	{
		return eventdetail;
	}

	/**
	 * @param i
	 */
	public void setEventdetail(int i)
	{
		eventdetail = i;
	}

	public static String getEventDetailString(int detail)
	{

		switch (detail)
		{
			case ESED_AUTHENTICATED :
				return (ESEDS_AUTHENTICATED);
			case ESED_BAD_CREDENTIALS :
				return (ESEDS_BAD_CREDENTIALS);
			case ESED_BAD_EAP_TYPE :
				return (ESEDS_BAD_EAP_TYPE);
			case ESED_INTERNAL_ERROR :
				return (ESEDS_INTERNAL_ERROR);
			case ESED_MSCHAPV2 :
				return (ESEDS_MSCHAPV2);
			case ESED_PEAP_PROTOCOL_FAILURE :
				return (ESEDS_PEAP_PROTOCOL_FAILURE);
			case ESED_REAUTHENTICATED :
				return (ESEDS_REAUTHENTICATED);
			default :
				return (ESEDS_NONE);
		}
	}

}
