/*
* Created on Apr 23, 2003
*
* To change the template for this generated file go to
* Window>Preferences>Java>Code Generation>Code and Comments
*/
package com.instant802.cldc.ssl;

import com.instant802.cldc.net.SSLSocket;
import com.instant802.cldc.net.SocketException;
import com.instant802.cldc.net.SocketTimeoutException;
import com.instant802.cldc.util.DateUtil;
import com.instant802.cldc.util.Log;

/**
* @author alan, robin, jgordon
*
* To change the template for this generated type comment go to
* Window>Preferences>Java>Code Generation>Code and Comments
*/
public class SSL
{
	protected int ssl = 0; // A place to stuff the SSL pointer
	protected SSLSessionContext ctx;
	
	public static final int SSL_FILETYPE_PEM = 1; // Same as X509_FILETYPE_PEM
	public static final int SSL_ERROR_WANT_READ = 2, SSL_ERROR_WANT_WRITE = 3;
	private static final int WRITE_BUF_SIZE = 4096;
	
	// John's lock idiom
	private int count = 0;
	private Object lock = new Boolean( true );
	private boolean closeMe = false;
	
	/*
* Turn off actual use of SSL in socket stuff. This is a short term
* useful hack that lets us turn off encryption in orchestration and see packets
* on the wire. Does not impact authentication encryption
*/
	//TODO: TURN SSL BACK ON!!
	public static final boolean encrypt = true;
	
	long timeout = 30000L; // timeout
	
	/**
* @param SSL_CONTEXT_MODE_SERVER
*/
	public SSL(SSLSessionContext ctx)
	{
		try{
			this.ctx = ctx;
			ssl = createSSL(ctx.getContextPtr());
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
		}
	}
	
	public void setTimeout(long timeout)
	{
		this.timeout = timeout;
	}
	
	private long endTime()
	{
		return DateUtil.now() + timeout;
	}
	
	
	private final void enterLock() throws SocketException {
		synchronized( lock ) {
			if( (ssl == 0) || closeMe ) {
				throw new SocketException("socket closed");
			}
			count++;
		}
	}
	
	private void exitLock() throws SocketException {
		synchronized( lock ) {
			count --;
			if( closeMe && ( count == 0 ) ) {
				actualClose();
				throw new SocketException("socket closed");
			}
		}
	}
	
	/**
* Blocking read built on top of non-blocking read which respects KVM threading.
* @param buf
* @param len
* @return >0 bytes read. < 0 if error

*/
	public  int read(SSLSocket socket, byte[] buf, int len) throws SocketException
	{
		try{
			int read = 0;
			long end = endTime();
			
			while (true)
			{
				do {
					try{
						enterLock();
						socket.checkOpen();
						read = read(ssl, buf, len, (int) timeout);
					} finally {
						exitLock();
					}
					if (-SSL_ERROR_WANT_WRITE == read) {
						try {Thread.sleep(100); } catch (Exception _) {}
					} else
						break;
				} while (true);
				
				if (read == 0) {
					throw new SocketTimeoutException("timed out in SSL read");
				} else if (-SSL_ERROR_WANT_READ == read) {
					read = 0;
					if (DateUtil.now() > end)
						throw new SocketTimeoutException("timed out in SSL read (libssl)");
					try {Thread.sleep(100); } catch (Exception _) {}
				} else
					break;
			}
			return read;
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return 0;
		}
		
	}
	
	/**
* @param buf
* @param len
* @return >0 bytes written. < 0 if error
*/
	
	public int write(SSLSocket socket, byte[] buf, int len) throws SocketException {
		if (len <= WRITE_BUF_SIZE) {
			return write0(socket, buf, len);
		} else {
			byte[] buffer = new byte[WRITE_BUF_SIZE];
			int left = len, start = 0, sent, size;
			
			while (left > 0) {
				size = (left > WRITE_BUF_SIZE) ? WRITE_BUF_SIZE : left;
				System.arraycopy(buf, start, buffer, 0, size);
				sent = write0(socket, buffer, size);
				if (sent <= 0)
					return sent;
				start = start +sent;
				left = left - sent;
			}
			return len;
		}
		
	}
	
	private int write0(SSLSocket socket, byte[] buf, int len) throws SocketException
	{
		try{
			int written = 0;
			long end = endTime();
			
			while (true)
			{
				do {
					try{
						enterLock();
						socket.checkOpen();
						written = write(ssl, buf, len);
					} finally {
						exitLock();
					}
					if (-SSL_ERROR_WANT_READ == written) {
						try {Thread.sleep(100); } catch (Exception _) {}
					} else
						break;
				} while (true);
				
				if (-SSL_ERROR_WANT_WRITE == written )
				{
					written = 0;
					if (DateUtil.now() > end)
						throw new SocketTimeoutException("timed out in SSL write (libssl)");
					try {Thread.sleep(100); } catch (Exception _) {}
				} else if (0 == written) {
					throw new SocketTimeoutException("timed out in SSL write");
				} else
					break;
			}
			return written;
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return 0;
		}
		
	}
	
	public int useRSAPrivateKey(String file, int type) throws SocketException
	{
		try{
			try {
				enterLock();
				return useRSAPrivateKey(ssl, file.getBytes(), file.length(), type);
			} finally {
				exitLock();
			}
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return 0;
		}
		
	}
	
	public int useCertificateFile(String file, int type) throws SocketException
	{
		try{
			try{
				enterLock();
				return useCertificateFile(ssl, file.getBytes(), file.length(), type);
			} finally {
				exitLock();
			}
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return 0;
		}
		
	}
	
	/**
* This should only ever be called in blocks synchronized on 'lock'
*/
	private final void actualClose() {
		try{
			int ssl = this.ssl;
			this.ssl = 0;
			close( ssl );
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
		}
	}
	
	public void close()
	{
		synchronized( lock ) {
			if( count == 0 ) {
				actualClose();
			} else {
				closeMe = true;
			}
		}
	}
	
	public int getError(int retcode) throws SocketException
	{
		try{
			enterLock();
			return getError(ssl, retcode);
		} finally {
			exitLock();
		}
	}
	
	public byte[] getMasterKey() throws SocketException
	{
		byte[] key;
		
		try{
			enterLock();
			key = new byte[getMasterKeySize(ssl)];
			getMasterKey(ssl, key, key.length);
		} finally {
			exitLock();
		}
		return key;
	}
	
	public byte[] getClientRandom() throws SocketException
	{
		try{
			byte[] rand;
			try{
				enterLock();
				rand = new byte[getRandomSize()];
				getClientRandom(ssl, rand, rand.length);
			} finally {
				exitLock();
			}
			return rand;
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return new byte[0];
		}
		
	}
	
	public byte[] getServerRandom() throws SocketException
	{
		try{
			byte[] rand;
			
			try{
				enterLock();
				rand = new byte[getRandomSize()];
				getServerRandom(ssl, rand, rand.length);
			} finally {
				exitLock();
			}
			return rand;
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return new byte[0];
		}
		
	}
	
	public void getRecvSendKeys(byte[] recv, byte[] send)
	throws SSLParameterException, SocketException
	{
		
		if (32 != recv.length || 32 != send.length || 0 == ssl)
			throw new SSLParameterException();
		try{
			enterLock();
			getRecvSendKeys(ssl, recv, send);
		} finally {
			exitLock();			
		}
	}
	
	public boolean setFd(int sd) throws SocketException
	{
		try{
			try{
				enterLock();
				return (1 == setFd(ssl, sd));
			} finally {
				exitLock();
			}
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return false;
		}
		
	}
	
	public void accept() throws SocketException
	{
		try{
			int val;
			
			try{
				enterLock();
				val = accept(ssl);
			} finally {
				exitLock();
			}
			if (1 != val)
			{
				throw new SocketException("SSL accept error: " + val);
			}
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
		}
		
	}
	
	public boolean connect() throws SocketException
	{
		try{		int val;
		
		try{
			enterLock();
			val = connect(ssl);
		} finally {
			exitLock();
		}
		
		if (1 != val)
		{
			throw new SocketException("SSL connect error: " + val);
		}
		return true;
		} catch ( NullPointerException e ) { 
			exitOnNullPointer( e );
			return false;
		}
		
	}
	
	private void exitOnNullPointer( NullPointerException e ) {
		try{
			Log.emergency( "SocketBase", e );
		} finally {
			System.exit( 1 );
		}
	}
	
	
	private native int createSSL(int ctxptr);
	
	private native int accept(int ssl);
	
	private native int connect(int ssl);
	
	private native int read(int ssl, byte[] buf, int buflen, int timeout);
	
	private native int write(int ssl, byte[] buf, int buflen);
	
	private native int useRSAPrivateKey(
			int ssl,
			byte[] filen,
			int filenlen,
			int type);
	
	private native int useCertificateFile(
			int ssl,
			byte[] filen,
			int filenlen,
			int type);
	
	private native void close(int ssl);
	
	private native int getMasterKeySize(int ssl);
	
	private native void getMasterKey(int ssl, byte[] key, int size);
	
	public native int getRandomSize();
	
	private native void getClientRandom(int ssl, byte[] key, int size);
	
	private native void getServerRandom(int ssl, byte[] key, int size);
	
	private native void getRecvSendKeys(int ssl, byte[] recv, byte[] send);
	
	private native int getError(int ssl, int retcode);
	
	private native int setFd(int ssl, int socketd);
}
